#include "pent editor dialogs.h"
#include "pent editor lists.h"
#include "pent editor files.h"
#include "pent editor dispatch.h"
#include "pent editor window.h"
#include "pent globals.h"
#include "dialog layer.h"
#include "key layer.h"
#include "error.h"
#include "main.h"
#include "menus.h"
#include "window layer.h"
#include "memory layer.h"
#include "program globals.h"

#define kGroupDialogID		401
enum { kGroupOKItem=1, kGroupCancelItem, kGroupUserItem, kGroupLabel, kGroupTextItem };

#define kBoardDialogID		410
enum { kBoardOKItem=1, kBoardCancelItem, kBoardUserItem, kBoardRowsItem, kBoardColumnsItem,
	kBoardLabelItem, kBoardTextItem };

#define kPieceSizeDialogID	405
enum { kPieceOKItem=1, kPieceCancelItem, kPieceUserItem, kPieceSizeItem };

#define kDeleteAlert		403

static	Boolean DoGroupDialogDispatch(Boolean isEdit, Str255 oldStr, Str255 newStr);
static	Boolean DoBoardDialogDispatch(Boolean isEdit, MyBoardPtr boardPtr);
static	pascal Boolean GroupModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem);
static	pascal Boolean BoardModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem);
static	pascal Boolean PieceSizeModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem);
static	pascal Boolean DeleteModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem);
static	Boolean CheckDeleteDialogDispatch(Boolean isGroup, Str255 theName);

Boolean OKToChangeBoardQQ(WindowRef theWindow, Str255 verb)
{
	short			result;
	
	if (!WindowIsModifiedQQ(theWindow))
		return TRUE;
	
	result=DisplayTheAlert(kCautionAlert, kSaveAlertID, "\pthe board ", gCurrentBoardRec.title,
		"\p", verb, (UniversalProcPtr)SaveModalFilter);
	switch (result)
	{
		case 1:
			DoSaveBoardDispatch(theWindow);
			return !WindowIsModifiedQQ(theWindow);
		case 2:
			return FALSE;
		case 3:
			return TRUE;
	}
	
	return TRUE;
}

void DoDeleteGroup(WindowRef theWindow)
{
	OSErr			oe;
	Str255			theName;
	short			highlightedIndex;
	
	if ((highlightedIndex=GetHighlightedGroup())<0)
		return;
	
	GetHighlightedGroupName(theName);
	
	if (!CheckDeleteDialogDispatch(TRUE, theName))
		return;
	
	if ((oe=DeleteOneGroup(theName))!=noErr)
	{
		HandleError(kCantDeleteGroup, FALSE, FALSE);
		return;
	}
	
	RebuildBoardMenu();
	DeleteHighlightedGroup();
	DoChangeDispatch(theWindow, TRUE);
}

void DoDeleteBoard(WindowRef theWindow)
{
	OSErr			oe;
	Str255			groupName, theName;
	short			highlightedIndex;
	
	if ((highlightedIndex=GetHighlightedBoard())<0)
		return;
	
	if (HighlightedBoardLockedQQ())
	{
		HandleError(kCantDeleteLockedBoard, FALSE, TRUE);
		return;
	}
	
	GetHighlightedBoardName(theName);
	if (!CheckDeleteDialogDispatch(FALSE, theName))
		return;
	
	GetHighlightedGroupName(groupName);
	if ((oe=DeleteOneBoard(groupName, highlightedIndex))!=noErr)
	{
		HandleError(kCantDeleteBoard, FALSE, FALSE);
		return;
	}
	
	RebuildBoardMenu();
	DeleteHighlightedBoard();
	DoChangeDispatch(theWindow, FALSE);
}

void DoNewGroup(WindowRef theWindow)
{
	Str255			groupName;
	OSErr			oe;
	
	if (!DoGroupDialogDispatch(FALSE, "\p", groupName))
		return;
	
	if (GroupNameExistsQQ(groupName))
	{
		HandleError(kDuplicateGroupName, FALSE, FALSE);
		return;
	}
	
	if ((oe=AddOneGroup(groupName))!=noErr)
	{
		HandleError(kCantCreateGroup, FALSE, FALSE);
		return;
	}
	
	RebuildBoardMenu();
	AddGroupToList(groupName, TRUE);
	DoChangeDispatch(theWindow, TRUE);
}

void DoNewBoard(WindowRef theWindow)
{
	Str255			groupName;
	OSErr			oe;
	MyBoardRec		boardRec;
	short			i, j;
	
	if (GetHighlightedGroup()<0)
		return;
	
	for (i=0; i<kBoardRowsMax; i++)
		for (j=0; j<kBoardColumnsMax; j++)
			boardRec.boardData[i][j]=kEmptyTile;
	boardRec.title[0]=0x00;
	boardRec.numRows=(gCurrentBoardRec.numRows==0) ? kBoardRowsDefault : gCurrentBoardRec.numRows;
	boardRec.numColumns=(gCurrentBoardRec.numRows==0) ? kBoardColumnsDefault : gCurrentBoardRec.numColumns;
	boardRec.locked=FALSE;
	boardRec.unused=FALSE;
	
	if (!DoBoardDialogDispatch(FALSE, &boardRec))
		return;
	
	GetHighlightedGroupName(groupName);
	
	if ((oe=AddOneBoard(groupName, &boardRec))!=noErr)
	{
		HandleError(kCantCreateBoard, FALSE, FALSE);
		return;
	}
	
	RebuildBoardMenu();
	AddBoardToList(boardRec.title, boardRec.locked, TRUE);
	DoChangeDispatch(theWindow, FALSE);
}

void DoEditGroup(WindowRef theWindow)
{
	#pragma unused(theWindow)
	
	Str255			oldStr, newStr;
	OSErr			oe;
	short			highlightedIndex;
	
	if ((highlightedIndex=GetHighlightedGroup())<0)
		return;
	
	GetHighlightedGroupName(oldStr);
	
	if (!DoGroupDialogDispatch(TRUE, oldStr, newStr))
		return;
	
	if (GroupNameExistsQQ(newStr))
	{
		HandleError(kDuplicateGroupName, FALSE, FALSE);
		return;
	}
	
	if ((oe=ReplaceOneGroup(oldStr, newStr))!=noErr)
	{
		HandleError(kCantReplaceGroup, FALSE, FALSE);
		return;
	}
	
	RebuildBoardMenu();
	EditGroupInList(highlightedIndex, newStr);
}

void DoEditBoard(WindowRef theWindow)
{
	Str255			groupName;
	OSErr			oe;
	short			boardIndex;
	MyBoardRec		boardRec;
	
	if (GetHighlightedGroup()<0)
		return;
	
	GetHighlightedGroupName(groupName);
	boardIndex=GetHighlightedBoard();
	if (boardIndex<0)
		return;
	
	if (HighlightedBoardLockedQQ())
	{
		HandleError(kCantEditLockedBoard, FALSE, TRUE);
		return;
	}
	
	Mymemcpy((Ptr)&boardRec, (Ptr)&gCurrentBoardRec, sizeof(MyBoardRec));
	if (!DoBoardDialogDispatch(TRUE, &boardRec))
		return;
	
	if ((oe=ReplaceOneBoard(groupName, boardIndex, &boardRec))!=noErr)
	{
		HandleError(kCantReplaceBoard, FALSE, FALSE);
		return;
	}
	
	Mymemcpy((Ptr)&gCurrentBoardRec, (Ptr)&boardRec, sizeof(MyBoardRec));
	RebuildBoardMenu();
	EditBoardInList(boardIndex, gCurrentBoardRec.title, gCurrentBoardRec.locked);
	DoChangeDispatch(theWindow, FALSE);
}

static	Boolean DoGroupDialogDispatch(Boolean isEdit, Str255 oldStr, Str255 newStr)
/* returns FALSE if cancelled or unchanged */
{
	DialogPtr		theDialog;
	UniversalProcPtr	modalFilterProc, userItemProc;
	Boolean			changed;
	Str255			theStr;
	short			itemSelected;
	
	theDialog=SetupTheDialog(kGroupDialogID, kGroupUserItem,
		isEdit ? "\pEdit group" : "\pNew group", (UniversalProcPtr)GroupModalFilter,
		&modalFilterProc, &userItemProc);
	SetDialogItemString(theDialog, kGroupTextItem, oldStr);
	HighlightDialogTextItem(theDialog, kGroupTextItem, 0, 32767);
	DisplayTheDialog(theDialog, TRUE);
	
	itemSelected=0;
	while ((itemSelected!=kGroupOKItem) && (itemSelected!=kGroupCancelItem))
	{
		itemSelected=GetOneDialogEvent(modalFilterProc);
	}
	
	changed=FALSE;
	
	if (itemSelected==kGroupOKItem)
	{
		GetDialogItemString(theDialog, kGroupTextItem, theStr);
		
		if (theStr[0]==0x00)
			changed=FALSE;
		else
		{
			changed=(!Mymemcompare((Ptr)oldStr, (Ptr)theStr, oldStr[0]+1));
			if (changed)
			{
				Mymemcpy(newStr, theStr, theStr[0]+1);	/* return new name */
			}
		}
	}
	
	ShutDownTheDialog(theDialog, modalFilterProc, userItemProc);
	
	while (HandleSingleEvent(FALSE)) {};
	
	return changed;
}

static	Boolean DoBoardDialogDispatch(Boolean isEdit, MyBoardPtr boardPtr)
/* returns FALSE if cancelled or unchanged; if returns TRUE, boardPtr is updated appropriately */
{
	DialogPtr		theDialog;
	UniversalProcPtr	modalFilterProc, userItemProc;
	Boolean			changed;
	Str255			theStr;
	short			itemSelected;
	short			numRows, numColumns;
	
	theDialog=SetupTheDialog(kBoardDialogID, kBoardUserItem,
		isEdit ? "\pEdit board" : "\pNew board", (UniversalProcPtr)BoardModalFilter,
		&modalFilterProc, &userItemProc);
	SetDialogItemString(theDialog, kBoardTextItem, boardPtr->title);
	SetDialogControlInfo(theDialog, kBoardRowsItem, kBoardRowsMin, kBoardRowsMax,
		boardPtr->numRows);
	SetDialogControlInfo(theDialog, kBoardColumnsItem, kBoardColumnsMin, kBoardColumnsMax,
		boardPtr->numColumns);
	HighlightDialogTextItem(theDialog, kBoardTextItem, 0, 32767);
	DisplayTheDialog(theDialog, TRUE);
	
	itemSelected=0;
	while ((itemSelected!=kBoardOKItem) && (itemSelected!=kBoardCancelItem))
	{
		itemSelected=GetOneDialogEvent(modalFilterProc);
	}
	
	changed=FALSE;
	
	if (itemSelected==kBoardOKItem)
	{
		GetDialogItemString(theDialog, kBoardTextItem, theStr);
		numRows=GetDialogControlValue(theDialog, kBoardRowsItem);
		numColumns=GetDialogControlValue(theDialog, kBoardColumnsItem);
		
		if (theStr[0]==0x00)
			changed=FALSE;
		else
		{
			if (theStr[0]>31)
				theStr[0]=31;
			changed=(!Mymemcompare((Ptr)boardPtr->title, (Ptr)theStr, theStr[0]+1));
			changed|=(numRows!=boardPtr->numRows);
			changed|=(numColumns!=boardPtr->numColumns);
			if (changed)
			{
				Mymemcpy((Ptr)boardPtr->title, theStr, theStr[0]+1);	/* return new name */
				boardPtr->numRows=numRows;
				boardPtr->numColumns=numColumns;
			}
		}
	}
	
	ShutDownTheDialog(theDialog, modalFilterProc, userItemProc);
	
	while (HandleSingleEvent(FALSE)) {};
	
	return changed;
}

Boolean DoPieceSizeDialog(void)
/* returns TRUE if changed */
{
	DialogPtr		theDialog;
	UniversalProcPtr	modalFilterProc, userItemProc;
	Boolean			changed;
	short			itemSelected;
	short			pieceSize;
	
	theDialog=SetupTheDialog(kPieceSizeDialogID, kPieceUserItem, "\pSet tile size",
		(UniversalProcPtr)PieceSizeModalFilter, &modalFilterProc, &userItemProc);
	SetDialogControlInfo(theDialog, kPieceSizeItem, kPieceSizeMin, kPieceSizeMax,
		gTileSize);
	DisplayTheDialog(theDialog, TRUE);
	
	itemSelected=0;
	while ((itemSelected!=kPieceOKItem) && (itemSelected!=kPieceCancelItem))
	{
		itemSelected=GetOneDialogEvent(modalFilterProc);
	}
	
	changed=FALSE;
	
	if (itemSelected==kPieceOKItem)
	{
		pieceSize=GetDialogControlValue(theDialog, kPieceSizeItem);
		changed=(pieceSize!=gTileSize);
		gTileSize=pieceSize;
	}
	
	ShutDownTheDialog(theDialog, modalFilterProc, userItemProc);
	
	while (HandleSingleEvent(FALSE)) {};
	
	return changed;
}

static	pascal Boolean GroupModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem)
{
	return PoorMansFilterProc(theDialog, theEvent, theItem, 0L);
}

static	pascal Boolean BoardModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem)
{
	return PoorMansFilterProc(theDialog, theEvent, theItem, 0L);
}

static	pascal Boolean PieceSizeModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem)
{
	return PoorMansFilterProc(theDialog, theEvent, theItem, 0L);
}

static	pascal Boolean DeleteModalFilter(DialogPtr theDialog, EventRecord *theEvent, short *theItem)
{
	if (PoorMansFilterProc(theDialog, theEvent, theItem, 0L))
		return TRUE;
	if (CheckCommandKey(theDialog, theEvent, theItem, 'D', 1))
		return TRUE;
	if (CheckCommandKey(theDialog, theEvent, theItem, 'C', 2))
		return TRUE;
	
	return FALSE;
}

static	Boolean CheckDeleteDialogDispatch(Boolean isRef, Str255 theName)
{
	ModalFilterUPP	procFilter = NewModalFilterProc(TwoButtonFilter);
	short			result;
	
	result=DisplayTheAlert(kCautionAlert, kDeleteAlert,
		isRef ? "\pgroup" : "\pboard", theName, "\p", "\p",
		(UniversalProcPtr)DeleteModalFilter);
	while (HandleSingleEvent(FALSE)) {};
	
	return (result==1);
}
