#include "pent editor dispatch.h"
#include "pent editor dialogs.h"
#include "pent editor files.h"
#include "pent editor lists.h"
#include "pent editor window.h"
#include "pent globals.h"
#include "menus.h"
#include "memory layer.h"
#include "error.h"
#include "window layer.h"
#include "program globals.h"

void DoNewDispatch(WindowRef theWindow, Boolean isGroup)
{
	if (isGroup)
	{
		if (OKToChangeBoardQQ(theWindow, "\pcreating a new group"))
			DoNewGroup(theWindow);
	}
	else
	{
		if (OKToChangeBoardQQ(theWindow, "\pcreating another"))
			DoNewBoard(theWindow);
	}
}

void DoEditDispatch(WindowRef theWindow)
{
	if (GroupListActiveQQ())
		DoEditGroup(theWindow);
	else
		DoEditBoard(theWindow);
}

void DoDeleteDispatch(WindowRef theWindow)
{
	if (GroupListActiveQQ())
		DoDeleteGroup(theWindow);
	else
		DoDeleteBoard(theWindow);
}

void DoClearDispatch(WindowRef theWindow)
{
	short			i, j;
	Boolean			anyChanges;
	
	if (GetHighlightedBoard()<0)
		return;
	if (HighlightedBoardLockedQQ())
	{
		HandleError(kCantClearLockedBoard, FALSE, TRUE);
		return;
	}
	
	anyChanges=WindowIsModifiedQQ(theWindow);
	for (i=0; i<kBoardRowsMax; i++)
	{
		for (j=0; j<kBoardColumnsMax; j++)
		{
			if (gCurrentBoardRec.boardData[i][j]!=kEmptyTile)
				anyChanges=TRUE;
			gCurrentBoardRec.boardData[i][j]=kEmptyTile;
		}
	}
	SetWindowIsModified(theWindow, anyChanges);
	InvalRect(&(theWindow->portRect));
}

Boolean DoSaveBoardDispatch(WindowRef theWindow)
/* returns TRUE if no problems, FALSE if error trying to save board */
{
	Str255			groupName;
	short			boardIndex;
	
	boardIndex=GetHighlightedBoard();
	if (boardIndex<0)
		return FALSE;
	GetHighlightedGroupName(groupName);
	if (ReplaceOneBoard(groupName, boardIndex, &gCurrentBoardRec)!=noErr)
	{
		HandleError(kCantReplaceBoard, FALSE, FALSE);
		return FALSE;
	}
	
	SetWindowIsModified(theWindow, FALSE);
	return TRUE;
}

void DoMarkLockedDispatch(WindowRef theWindow)
{
	gCurrentBoardRec.locked=!gCurrentBoardRec.locked;
	if (DoSaveBoardDispatch(theWindow))
		SetHighlightedBoardLocked(gCurrentBoardRec.locked);
	else
		gCurrentBoardRec.locked=!gCurrentBoardRec.locked;
}

void DoShowMessageDispatch(WindowRef theWindow)
{
	gShowMessageBox=!gShowMessageBox;
	EraseRect(&(theWindow->portRect));
	ResizeTheEditorWindow(theWindow);
	ResizeTheEditorBoard(theWindow);
}

void DoShowToolbarDispatch(WindowRef theWindow)
{
	gShowToolbar=!gShowToolbar;
	EraseRect(&(theWindow->portRect));
	ResizeTheEditorWindow(theWindow);
	ResizeTheEditorBoard(theWindow);
}

void DoChangeDispatch(WindowRef theWindow, Boolean changeGroup)
{
	Str255			groupName;
	short			boardIndex;
	OSErr			oe;
	Point			thePoint;
	MyBoardRec		boardRec;
	
	if ((GetHighlightedGroup()<0) || ((boardIndex=GetHighlightedBoard())<0))
	{
		gCurrentBoardRec.numRows=0;
		SetWindowIsModified(theWindow, FALSE);
	}
	else
	{
		GetHighlightedGroupName(groupName);
		if ((oe=GetOneBoard(groupName, boardIndex, &boardRec))!=noErr)
		{
			HandleError(kCantLoadBoard, FALSE, FALSE);
			return;
		}
		Mymemcpy((Ptr)&gCurrentBoardRec, (Ptr)&boardRec, sizeof(MyBoardRec));
		SetWindowIsModified(theWindow, FALSE);
	}
	
	if (changeGroup)
		BuildBoardListHandle(GetBoardsFileFSPtr(), TRUE);
	ResizeTheEditorBoard(theWindow);
	GetMouse(&thePoint);
	SetTheEditorMessage(theWindow, 0, FALSE);
	IdleInEditorWindow(theWindow, thePoint);
	InvalRect(&(theWindow->portRect));
}
