#include "pent editor files.h"
#include "pent editor window.h"
#include "pent editor lists.h"
#include "pent main window.h"
#include "pent globals.h"
#include "error.h"
#include "file interface.h"
#include "file utilities.h"
#include "memory layer.h"
#include "menus.h"
#include "resource layer.h"
#include "window layer.h"
#include "program globals.h"

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct MyMenuQ
{
	struct MyMenuQ	*next;
	Str31			groupName;
	MenuHandle		subMenu;
	short			subMenuID;
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef struct MyMenuQ		MyMenuQ, *MyMenuQPtr;

static	QHdr			gMenuQ;
static	FSSpec			gBoardsFileFS;

void InitBoardsFileFS(void)
{
	unsigned char	*boardsFileName=BOARDS_FILE_NAME;
	short			curResFile;
	short			oldRefNum, refNum, resID, i, numDefaultBoards;
	Boolean			alreadyOpen;
	Handle			resHandle;
	ResType			resType;
	Str255			groupName;
	
	GetApplicationFSSpec(&gBoardsFileFS);
	Mymemcpy((Ptr)gBoardsFileFS.name, (Ptr)boardsFileName, boardsFileName[0]+1);
	if (FileExists(&gBoardsFileFS)!=noErr)
	{
		FSpCreateResFile(&gBoardsFileFS, CREATOR, BOARD_FILE_TYPE, smSystemScript);
		if (ResError()!=noErr)
			HandleError(kCantCreateBoardsFile, TRUE, FALSE);
		else
		{	// ought to have some error checking in here -- if anything goes wrong, delete new file
			curResFile=CurResFile();
			numDefaultBoards=Count1Resources(kBoardResType);
			OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE);
			for (i=1; i<=numDefaultBoards; i++)
			{
				UseResFile(curResFile);
				resHandle=Get1IndResource(kBoardResType, i);
				GetResInfo(resHandle, &resID, &resType, groupName);
				DetachResource(resHandle);
				UseResFile(refNum);
				AddResource(resHandle, kBoardResType, resID, groupName);
				WriteResource(resHandle);
				UpdateResFile(refNum);
			}
			CloseTheResFile(oldRefNum, refNum, alreadyOpen);
			HandleError(kCantFindBoardsFile, FALSE, FALSE);
		}
	}
}

FSSpec* GetBoardsFileFSPtr(void)
{
	return &gBoardsFileFS;
}

void SetGroupAndBoard(short menuID, short menuItem)
{
	MyMenuQPtr		q;
	
	for (q=(MyMenuQPtr)(gMenuQ.qHead); q!=0L; q=q->next)
	{
		if (q->subMenuID==menuID)
		{
			Mymemcpy((Ptr)gCurrentGroupName, (Ptr)q->groupName, q->groupName[0]+1);
			gCurrentBoardIndex=menuItem-1;
			return;
		}
	}
}

void SetBoardCheckmark(void)
{
	MyMenuQPtr		q;
	short			i, numBoards;
	
	for (q=(MyMenuQPtr)(gMenuQ.qHead); q!=0L; q=q->next)
	{
		numBoards=CountMItems(q->subMenu);
		for (i=1; i<=numBoards; i++)
		{
			CheckItem(q->subMenu, i, FALSE);
		}
		if (Mymemcompare((Ptr)q->groupName, (Ptr)gCurrentGroupName, gCurrentGroupName[0]+1))
		{
			CheckItem(q->subMenu, gCurrentBoardIndex+1, TRUE);
		}
	}
}

OSErr BuildBoardMenu(MenuHandle theMenu)
{
	OSErr			oe;
	short			oldRefNum, refNum, resID, numGroups, numBoards, i, j, subMenuID;
	Boolean			alreadyOpen;
	ResType			resType;
	Handle			resHandle;
	Str255			groupName;
	MenuHandle		subMenu;
	MyBoardRec		boardRec;
	MyMenuQPtr		q;
	unsigned char	markChar;
	Boolean			currentGroupOK, currentBoardOK, thisIsCurrentGroup;
	
	for (q=(MyMenuQPtr)(gMenuQ.qHead); q!=0L; q=(MyMenuQPtr)(gMenuQ.qHead))
	{
		DeleteMenu(q->subMenuID);
		DisposeMenu(q->subMenu);
		Dequeue((QElemPtr)q, &gMenuQ);
	}
	while (CountMItems(theMenu)>0)
		DeleteMenuItem(theMenu, 1);
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, TRUE))!=noErr)
		return oe;
	
	numGroups=Count1Resources(kBoardResType);
	currentGroupOK=currentBoardOK=FALSE;
	for (i=1; i<=numGroups; i++)
	{
		resHandle=Get1IndResource(kBoardResType, i);
		if (resHandle==0L)
		{
			oe=ResError();
			CloseTheResFile(oldRefNum, refNum, alreadyOpen);
			return oe;
		}
		
		GetResInfo(resHandle, &resID, &resType, groupName);
		oe=ResError();
		if (oe!=noErr)
		{
			ReleaseResource(resHandle);
			CloseTheResFile(oldRefNum, refNum, alreadyOpen);
			return oe;
		}
		AppendMenu(theMenu, "\p ");
		SetItem(theMenu, CountMItems(theMenu), groupName);
		thisIsCurrentGroup=FALSE;
		if (Mymemcompare((Ptr)groupName, (Ptr)gCurrentGroupName, gCurrentGroupName[0]+1))
			thisIsCurrentGroup=currentGroupOK=TRUE;
		subMenuID=149+i;
		markChar=subMenuID;
		SetItemCmd(theMenu, CountMItems(theMenu), hMenuCmd);
		SetItemMark(theMenu, CountMItems(theMenu), (char)markChar);
		subMenu=NewMenu(subMenuID, "\p ");
		numBoards=GetHandleSize(resHandle)/sizeof(MyBoardRec);
		for (j=0; j<numBoards; j++)
		{
			Mymemcpy((Ptr)&boardRec, (Ptr)((unsigned long)*resHandle+sizeof(MyBoardRec)*j),
				sizeof(MyBoardRec));
			if ((boardRec.title[0]==0x01) && (boardRec.title[1]=='-'))
				AppendMenu(subMenu, boardRec.title);
			else
			{
				AppendMenu(subMenu, "\p ");
				SetItem(subMenu, CountMItems(subMenu), boardRec.title);
			}
			if (thisIsCurrentGroup)
				if (gCurrentBoardIndex==j)
					currentBoardOK=TRUE;
		}
		if (numBoards==0)
		{
			AppendMenu(subMenu, "\p(No boards");
			if (thisIsCurrentGroup)
				currentGroupOK=FALSE;
		}
		else
		{
			if (thisIsCurrentGroup && !currentBoardOK)
			{
				gCurrentBoardIndex=0;
				currentBoardOK=TRUE;
			}
		}
		InsertMenu(subMenu, -1);
		q=(MyMenuQPtr)NewPtrClear(sizeof(MyMenuQ));
		q->subMenu=subMenu;
		q->subMenuID=subMenuID;
		Mymemcpy((Ptr)q->groupName, (Ptr)groupName, groupName[0]+1);
		Enqueue((QElemPtr)q, &gMenuQ);
		ReleaseResource(resHandle);
		resHandle=0L;
	}
	
	if (numGroups==0)
	{
		AppendMenu(theMenu, "\p(No boards");
		gCurrentBoardIndex=-1;
	}
	else if (!currentGroupOK)
	{
		gCurrentBoardIndex=-1;
		for (i=1; ((i<=numGroups) && (gCurrentBoardIndex==-1)); i++)
		{
			resHandle=Get1IndResource(kBoardResType, 1);
			if (GetHandleSize(resHandle)!=0L)
			{
				GetResInfo(resHandle, &resID, &resType, groupName);
				Mymemcpy((Ptr)gCurrentGroupName, (Ptr)groupName, groupName[0]+1);
				gCurrentBoardIndex=0;
			}
			ReleaseResource(resHandle);
			resHandle=0L;
		}
	}
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return noErr;
}

Boolean GroupNameExistsQQ(Str255 groupName)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	Boolean			exists;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, TRUE))!=noErr)
		return FALSE;
	
	resHandle=Get1NamedResource(kBoardResType, groupName);
	exists=(resHandle!=0L);
	if (exists)
		ReleaseResource(resHandle);
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return exists;
}

OSErr DeleteOneGroup(Str255 groupName)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, groupName);
	if (resHandle==0L)
		return ResError();
	
	RmveResource(resHandle);
	oe=ResError();
	if (oe==noErr)
	{
		UpdateResFile(refNum);
		DisposeHandle(resHandle);	/* correct, this handle is no longer a resource */
	}
	else ReleaseResource(resHandle);
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}

OSErr DeleteOneBoard(Str255 groupName, short boardIndex)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, groupName);
	if (resHandle!=0L)
	{
		Mymemcpy((Ptr)((unsigned long)*resHandle+sizeof(MyBoardRec)*boardIndex),
			(Ptr)((unsigned long)*resHandle+sizeof(MyBoardRec)*(boardIndex+1)),
			GetHandleSize(resHandle)-sizeof(MyBoardRec)*(boardIndex+1));
		SetHandleSize(resHandle, GetHandleSize(resHandle)-sizeof(MyBoardRec));
		ChangedResource(resHandle);
		WriteResource(resHandle);
		UpdateResFile(refNum);
		ReleaseResource(resHandle);
	}
	else oe=ResError();
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}

OSErr AddOneGroup(Str255 groupName)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=NewHandle(0L);
	AddResource(resHandle, kBoardResType, Unique1ID(kBoardResType), groupName);
	oe=ResError();
	
	if (oe==noErr)
	{
		WriteResource(resHandle);
		oe=ResError();
	}
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	ReleaseResource(resHandle);
	
	return oe;
}

OSErr AddOneBoard(Str255 groupName, MyBoardPtr boardPtr)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	long			oldSize;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, groupName);
	if (resHandle!=0L)
	{
		oldSize=GetHandleSize(resHandle);
		SetHandleSize(resHandle, oldSize+sizeof(MyBoardRec));
		Mymemcpy((Ptr)((unsigned long)*resHandle+oldSize), (Ptr)boardPtr, sizeof(MyBoardRec));
		ChangedResource(resHandle);
		WriteResource(resHandle);
		UpdateResFile(refNum);
		ReleaseResource(resHandle);
	}
	
	oe=ResError();
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}

OSErr ReplaceOneGroup(Str255 oldGroup, Str255 newGroup)
{
	OSErr			oe;
	short			oldRefNum, refNum, resID;
	Boolean			alreadyOpen;
	Str255			resName;
	ResType			resType;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, oldGroup);
	if (resHandle!=0L)
	{
		GetResInfo(resHandle, &resID, &resType, resName);
		SetResInfo(resHandle, resID, newGroup);
		oe=ResError();
		if (oe==noErr)
		{
			ChangedResource(resHandle);
			WriteResource(resHandle);
			UpdateResFile(refNum);
			oe=ResError();
		}
		ReleaseResource(resHandle);
	}
	else oe=ResError();
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}

OSErr ReplaceOneBoard(Str255 oldGroup, short boardIndex, MyBoardPtr boardPtr)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, oldGroup);
	if (resHandle!=0L)
	{
		Mymemcpy((Ptr)((unsigned long)*resHandle+sizeof(MyBoardRec)*boardIndex),
			(Ptr)boardPtr, sizeof(MyBoardRec));
		ChangedResource(resHandle);
		WriteResource(resHandle);
		UpdateResFile(refNum);
		oe=ResError();
		ReleaseResource(resHandle);
	}
	else oe=ResError();
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}

OSErr GetOneBoard(Str255 groupName, short boardIndex, MyBoardPtr boardPtr)
{
	OSErr			oe;
	short			oldRefNum, refNum;
	Boolean			alreadyOpen;
	Handle			resHandle;
	
	if ((oe=OpenTheResFile(&gBoardsFileFS, &oldRefNum, &refNum, &alreadyOpen, TRUE))!=noErr)
		return oe;
	
	resHandle=Get1NamedResource(kBoardResType, groupName);
	if (resHandle!=0L)
	{
		Mymemcpy((Ptr)boardPtr, (Ptr)((unsigned long)*resHandle+sizeof(MyBoardRec)*boardIndex),
			sizeof(MyBoardRec));
		ReleaseResource(resHandle);
		oe=noErr;
	}
	else oe=ResError();
	
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	
	return oe;
}
