#include "pent editor window.h"
#include "pent editor lists.h"
#include "pent editor files.h"
#include "pent editor dispatch.h"
#include "pent editor dialogs.h"
#include "pent graphics.h"
#include "pent globals.h"
#include "program globals.h"
#include "button layer.h"
#include "environment.h"
#include "error.h"
#include "key layer.h"
#include "window layer.h"
#include "menus.h"
#include "main.h"
#include "text layer.h"
#include "resource layer.h"
#include "memory layer.h"
#include "list layer.h"
#include <Icons.h>

#define DEAD_SPACE_H		10
#define DEAD_SPACE_V		10
#define BUTTON_GAP			1
#define BUTTON_WIDTH		24
#define BUTTON_HEIGHT		24
#define MESSAGE_HEIGHT		19
#define HEADER_SPACE		(DEAD_SPACE_V*3+BUTTON_HEIGHT+MESSAGE_HEIGHT)
#define HEADER_NO_MESSAGE	(DEAD_SPACE_V*2+BUTTON_HEIGHT)
#define HEADER_NO_TOOLBAR	0
#define kHeaderHeight		(gShowToolbar ? (gShowMessageBox ? HEADER_SPACE : HEADER_NO_MESSAGE) : HEADER_NO_TOOLBAR)
#define kGrowBoxSize		15

#define kFirstEditorButtonID	600
#define kEditorMessagesID		500
#define NUM_BUTTONS				9

enum { kNewGroupButton=0, kNewBoardButton, kEditButton, kDeleteButton,
		kClearButton, kLockButton, kSaveButton, kMessageButton, kToolbarButton };

enum { kNewGroupMessage=1, kNewBoardMessage, kNewBoardDimmedMessage, kEditGroupMessage,
		kEditGroupDimmedMessage, kEditBoardMessage, kEditBoardNoneHighlightedMessage,
		kEditBoardLockedMessage, kDeleteGroupMessage, kDeleteGroupDimmedMessage,
		kDeleteBoardMessage, kDeleteBoardNoneHighlightedMessage, kDeleteBoardLockedMessage,
		kClearBoardMessage, kClearBoardNoneHighlightedMessage, kClearBoardLockedMessage,
		kLockBoardMessage, kLockBoardDimmedMessage, kUnlockBoardMessage, kSaveBoardMessage,
		kSaveBoardNoChangesMessage, kSaveBoardNoneHighlightedMessage,
		kHideMessageMessage, kHideToolbarMessage, kFreeTilesMessage, kBlankMessage };

static	Boolean ButtonIsDimmedQQ(WindowRef theWindow, short buttonNum);
static	void DrawTheEditorMessage(WindowRef theWindow, Boolean drawFrame);
static	void DrawTheEditorBoard(WindowRef theWindow, short theDepth);
static	void DrawEditorHeader(WindowRef theWindow, short theDepth);
static	short FreeTiles(void);

static	Rect			gButtonRect[NUM_BUTTONS];
static	Handle			gButtonIcon[NUM_BUTTONS];	/* handle to icon suite */
static	Str255			gTheMessage;
static	short			gTheMessageID;
static	Boolean			gOldButtonState[NUM_BUTTONS];
static	Boolean			gSetupDone=FALSE;
static	short			gEditorTileSize;
static	Point			gEditorBoardTopLeft;

void SetupTheEditorWindow(WindowRef theWindow)
{
	unsigned char	*titleStr="\pPentominoes Board Editor";
	Point			topLeft;
	short			i;
	
	SetWindowHeight(theWindow, qd.screenBits.bounds.bottom-qd.screenBits.bounds.top-LMGetMBarHeight()-28);
	SetWindowWidth(theWindow, qd.screenBits.bounds.right-qd.screenBits.bounds.left-20);
	SetWindowAttributes(theWindow, kHasCloseBoxMask+kHasZoomBoxMask+kHasGrowBoxMask+kHasDocumentTitlebarMask);
	topLeft.v=qd.screenBits.bounds.top+LMGetMBarHeight()+20;
	topLeft.h=qd.screenBits.bounds.left+10;
	SetWindowTopLeft(theWindow, topLeft);
	SetWindowMaxDepth(theWindow, 8);
	SetWindowIsFloat(theWindow, FALSE);
	SetWindowIsZoomable(theWindow, TRUE);
	SetWindowIsPrintable(theWindow, TRUE);
	SetWindowTitle(theWindow, titleStr);
	SetWindowAutoCenter(theWindow, FALSE);
	SetWindowIsModified(theWindow, FALSE);
	SetWindowDrawGrowIconLines(theWindow, FALSE);
	
	gCurrentBoardRec.numRows=0;
	
	if (gSetupDone)
		return;
	
	gSetupDone=TRUE;
	gNeedToOpenWindow=FALSE;
	for (i=0; i<NUM_BUTTONS; i++)
	{
		SetRect(&gButtonRect[i], DEAD_SPACE_H+i*(BUTTON_WIDTH+BUTTON_GAP), DEAD_SPACE_V,
			DEAD_SPACE_H+i*(BUTTON_WIDTH+BUTTON_GAP)+BUTTON_WIDTH, DEAD_SPACE_V+BUTTON_HEIGHT);
		GetIconSuite(&gButtonIcon[i], kFirstEditorButtonID+i, svAllSmallData);
	}
	gTheMessageID=0;
	SetTheEditorMessage(theWindow, 0, FALSE);
	InitTheEditorLists();
}

void ShutDownTheEditorWindow(void)
{
	short			i;
	
	for (i=0; i<NUM_BUTTONS; i++)
		gButtonIcon[i]=SafeDisposeIconSuite(gButtonIcon[i]);
}

void OpenTheEditorWindow(WindowRef theWindow)
{
	theWindow->txFont=geneva;
	theWindow->txSize=9;
	
	SetEditorListRects(theWindow, DEAD_SPACE_H, DEAD_SPACE_V, kHeaderHeight);
	CreateEditorLists(theWindow);
	BuildGroupListHandle(GetBoardsFileFSPtr(), FALSE);
	SetGroupListActive(FALSE);
	SetWindowIsActive(theWindow, TRUE);
	AdjustMenus();
}

Boolean CloseTheEditorWindow(WindowRef theWindow)
{
	return OKToChangeBoardQQ(theWindow, gDone ? "\pquitting" : "\pclosing");
}

void IdleInEditorWindow(WindowRef theWindow, Point mouseLoc)
{
	short			i;
	Boolean			gotone;
	short			id;
	
	if (!gShowToolbar || !gShowMessageBox)
		return;
	
	if (!gIsInBackground)
	{
		for (i=0, gotone=FALSE; ((i<NUM_BUTTONS) && (!gotone)); i++)
		{
			if (PtInRect(mouseLoc, &gButtonRect[i]))
			{
				gotone=TRUE;
				switch (i)
				{
					case kNewGroupButton:
						id=kNewGroupMessage;
						break;
					case kNewBoardButton:
						id=(ButtonIsDimmedQQ(theWindow, i)) ? kNewBoardDimmedMessage : kNewBoardMessage;
						break;
					case kEditButton:
						id=(GroupListActiveQQ()) ?
							(ButtonIsDimmedQQ(theWindow, i)) ? kEditGroupDimmedMessage : kEditGroupMessage :
							(!ButtonIsDimmedQQ(theWindow, i)) ? kEditBoardMessage :
							(GetHighlightedBoard()<0) ? kEditBoardNoneHighlightedMessage : kEditBoardLockedMessage;
						break;
					case kDeleteButton:
						id=(GroupListActiveQQ()) ?
							(ButtonIsDimmedQQ(theWindow, i)) ? kDeleteGroupDimmedMessage : kDeleteGroupMessage :
							(!ButtonIsDimmedQQ(theWindow, i)) ? kDeleteBoardMessage :
							(GetHighlightedBoard()<0) ? kDeleteBoardNoneHighlightedMessage : kDeleteBoardLockedMessage;
						break;
					case kClearButton:
						id=(!ButtonIsDimmedQQ(theWindow, i)) ? kClearBoardMessage :
							(GetHighlightedBoard()<0) ? kClearBoardNoneHighlightedMessage : kClearBoardLockedMessage;
						break;
					case kLockButton:
						id=ButtonIsDimmedQQ(theWindow, i) ? kLockBoardDimmedMessage :
							(HighlightedBoardLockedQQ() ? kUnlockBoardMessage : kLockBoardMessage);
						break;
					case kSaveButton:
						id=(!ButtonIsDimmedQQ(theWindow, i)) ? kSaveBoardMessage :
							(GetHighlightedBoard()<0) ? kSaveBoardNoneHighlightedMessage : kSaveBoardNoChangesMessage;
						break;
					case kMessageButton:
						id=kHideMessageMessage;
						break;
					case kToolbarButton:
						id=kHideToolbarMessage;
						break;
				}
				if (id!=gTheMessageID)
					SetTheEditorMessage(theWindow, id, TRUE);
			}
		}
	}
	
	if (!gotone)
	{
		if ((gCurrentBoardRec.numRows==0) && (gTheMessageID!=kBlankMessage))
			SetTheEditorMessage(theWindow, kBlankMessage, TRUE);
		if ((gCurrentBoardRec.numRows>0) && (gTheMessageID!=kFreeTilesMessage))
			SetTheEditorMessage(theWindow, kFreeTilesMessage, TRUE);
	}
}

void KeyPressedInEditorWindow(WindowRef theWindow, unsigned char theChar)
{
	Boolean			groupActive;
	short			highlightedIndex, newIndex;
	short			max, numVisible;
	ListHandle		theList;
	Cell			theCell;
	
	groupActive=GroupListActiveQQ();
	theList=groupActive ? GetGroupListHandle() : GetBoardListHandle();
	highlightedIndex=groupActive ? GetHighlightedGroup() : GetHighlightedBoard();
	max=groupActive ? GetNumberOfGroups() : GetNumberOfBoards();
	numVisible=groupActive ? GetNumberOfVisibleGroups() : GetNumberOfVisibleBoards();
	newIndex=-2;
	
	RememberEditorButtonState(theWindow);
	
	switch (theChar)
	{
		case key_Tab:
			if (groupActive)
				SetBoardListActive(TRUE);
			else
				SetGroupListActive(TRUE);
			break;
		case key_UpArrow:
		case key_LeftArrow:
			newIndex=(highlightedIndex<=0) ? max-1 : highlightedIndex-1;
			/* no break intentionally */
		case key_DownArrow:
		case key_RightArrow:
			if (newIndex==-2)
				newIndex=((highlightedIndex==-1) || (highlightedIndex==max-1)) ? 0 : highlightedIndex+1;
			if (OKToChangeBoardQQ(theWindow, groupActive ? "\pviewing another group" : "\pediting another"))
			{
				SetPt(&theCell, 0, newIndex);
				MySelectOneCell(theList, theCell);
				DoChangeDispatch(theWindow, groupActive);
			}
			break;
		case key_Home:
			MyScrollList(theList, -max);
			break;
		case key_End:
			MyScrollList(theList, max);
			break;
		case key_PageUp:
			MyScrollList(theList, -numVisible);
			break;
		case key_PageDown:
			MyScrollList(theList, numVisible);
			break;
		case key_Return:
		case key_Enter:
			if (highlightedIndex>=0)
				DoEditDispatch(theWindow);
			break;
		case key_Delete:
			if (highlightedIndex>=0)
				DoDeleteDispatch(theWindow);
			break;
	}

	UpdateEditorButtons(theWindow, TRUE);
}

Boolean MouseClickedInEditorWindow(WindowRef theWindow, Point thePoint)
{
	Boolean			gotone;
	short			theDepth;
	Rect			groupListRect, boardListRect;
	ListHandle		groupList, boardList;
	Boolean			doubleClick;
	short			oldGroup, oldBoard, newGroup, newBoard;
	short			whichWay, i, j;
	Rect			boardRect;
	
	if (gInProgress)
		return TRUE;
	
	theDepth=GetWindowDepth(theWindow);
	RememberEditorButtonState(theWindow);
	
	for (i=0, gotone=FALSE; ((i<NUM_BUTTONS) && (!gotone)); i++)
	{
		if ((gShowToolbar) && (PtInRect(thePoint, &gButtonRect[i])) && (!ButtonIsDimmedQQ(theWindow, i)))
		{
			gotone=TRUE;
			if (Track3DButton(&gButtonRect[i], 0L, gButtonIcon[i], theDepth, kSquareButton))
			{
				switch (i)
				{
					case kNewGroupButton:
						DoNewDispatch(theWindow, TRUE);
						break;
					case kNewBoardButton:
						DoNewDispatch(theWindow, FALSE);
						break;
					case kEditButton:
						DoEditDispatch(theWindow);
						break;
					case kDeleteButton:
						DoDeleteDispatch(theWindow);
						break;
					case kClearButton:
						DoClearDispatch(theWindow);
						break;
					case kLockButton:
						DoMarkLockedDispatch(theWindow);
						break;
					case kSaveButton:
						DoSaveBoardDispatch(theWindow);
						break;
					case kMessageButton:
						DoShowMessageDispatch(theWindow);
						break;
					case kToolbarButton:
						DoShowToolbarDispatch(theWindow);
						break;
				}
			}
		}
	}
	
	if (!gotone)
	{
		groupList=GetGroupListHandle();
		boardList=GetBoardListHandle();
		GetGroupListRect(&groupListRect);
		GetBoardListRect(&boardListRect);
		
		if (PtInRect(thePoint, &groupListRect))
		{
			oldGroup=GetHighlightedGroup();
			SetGroupListActive(TRUE);
			doubleClick=MyHandleMouseDownInList(groupList, thePoint, GetTheModifiers());
			if (oldGroup!=GetHighlightedGroup())
			{
				newGroup=GetHighlightedGroup();
				if (WindowIsModifiedQQ(theWindow))
				{
					SetHighlightedGroup(oldGroup);
					if (OKToChangeBoardQQ(theWindow, "\pviewing another group"))
					{
						SetHighlightedGroup(newGroup);
						DoChangeDispatch(theWindow, TRUE);
					}
				}
				else DoChangeDispatch(theWindow, TRUE);
			}
			else if (doubleClick)
				DoEditDispatch(theWindow);
		}
		else if (PtInRect(thePoint, &boardListRect))
		{
			oldBoard=GetHighlightedBoard();
			SetBoardListActive(TRUE);
			doubleClick=MyHandleMouseDownInList(boardList, thePoint, GetTheModifiers());
			if (oldBoard!=GetHighlightedBoard())
			{
				newBoard=GetHighlightedBoard();
				if (WindowIsModifiedQQ(theWindow))
				{
					SetHighlightedBoard(oldBoard);
					if (OKToChangeBoardQQ(theWindow, "\pediting another"))
					{
						SetHighlightedBoard(newBoard);
						DoChangeDispatch(theWindow, FALSE);
					}
				}
				else DoChangeDispatch(theWindow, FALSE);
			}
			else if (doubleClick)
				DoEditDispatch(theWindow);
		}
	}
	
	if (!gotone)
	{
		boardRect.left=gEditorBoardTopLeft.h+8+3+1+5+1+1;
		boardRect.top=gEditorBoardTopLeft.v+3+1+5+1+1;
		boardRect.right=boardRect.left+(gCurrentBoardRec.numColumns*gEditorTileSize)-1;
		boardRect.bottom=boardRect.top+(gCurrentBoardRec.numRows*gEditorTileSize)-1;
		
		if (PtInRect(thePoint, &boardRect))
		{
			gotone=TRUE;
			if (HighlightedBoardLockedQQ())
			{
				HandleError(kCantChangeLockedBoard, FALSE, TRUE);
			}
			else
			{
				whichWay=0;
				
				while (StillDown())
				{
					GetMouse(&thePoint);
					if (PtInRect(thePoint, &boardRect))
					{
						GetRowColFromPoint(thePoint, &i, &j, gEditorTileSize, gEditorTileSize,
							gEditorBoardTopLeft.h, gEditorBoardTopLeft.v);
			
						switch (gCurrentBoardRec.boardData[i][j])
						{
							case kNoTile:
								if (whichWay>=0)
								{
									gCurrentBoardRec.boardData[i][j]=kEmptyTile;
									DrawOneSpecialTile(theWindow, theDepth, gCurrentBoardRec.boardData,
										i, j, gEditorTileSize, gEditorTileSize, gEditorBoardTopLeft.h,
										gEditorBoardTopLeft.v, FALSE);
									whichWay=1;
									SetWindowIsModified(theWindow, TRUE);
									SetTheEditorMessage(theWindow, kFreeTilesMessage, TRUE);
								}
								break;
							case kEmptyTile:
								if (whichWay<=0)
								{
									gCurrentBoardRec.boardData[i][j]=kNoTile;
									DrawOneSpecialTile(theWindow, theDepth, gCurrentBoardRec.boardData,
										i, j, gEditorTileSize, gEditorTileSize, gEditorBoardTopLeft.h,
										gEditorBoardTopLeft.v, FALSE);
									whichWay=-1;
									SetWindowIsModified(theWindow, TRUE);
									SetTheEditorMessage(theWindow, kFreeTilesMessage, TRUE);
								}
								break;
						}
					}
				}
			}
		}
	}
	
	UpdateEditorButtons(theWindow, TRUE);
	
	return gotone;
}

void DisposeTheEditorWindow(void)
{
	DisposeEditorLists();
}

void ActivateTheEditorWindow(WindowRef theWindow, Boolean drawGrowIconLines)
{
	ListHandle		groupList, boardList;
	
	groupList=GetGroupListHandle();
	boardList=GetBoardListHandle();
	LActivate(TRUE, groupList);
	LActivate(TRUE, boardList);
	MyDrawActiveListBorder(groupList, GroupListActiveQQ());
	MyDrawActiveListBorder(boardList, BoardListActiveQQ());
	DrawGrowIconNoLines(theWindow, drawGrowIconLines);
}

void DeactivateTheEditorWindow(WindowRef theWindow)
{
	Rect			tempRect;
	ListHandle		groupList, boardList;
	short			theDepth;
	
	theDepth=GetWindowDepth(theWindow);
	groupList=GetGroupListHandle();
	boardList=GetBoardListHandle();
	LActivate(FALSE, groupList);
	LActivate(FALSE, boardList);
	MyDrawActiveListBorder(groupList, FALSE);
	MyDrawActiveListBorder(boardList, FALSE);
	tempRect.bottom=theWindow->portRect.bottom;
	tempRect.right=theWindow->portRect.right;
	tempRect.top=tempRect.bottom-kGrowBoxSize+1;
	tempRect.left=tempRect.right-kGrowBoxSize+1;
	if (theDepth>2)
	{
		tempRect.left++;
		tempRect.top++;
	}
	EraseRect(&tempRect);
}

void ResizeTheEditorWindow(WindowRef theWindow)
{
	ResizeEditorLists(theWindow, DEAD_SPACE_H, DEAD_SPACE_V, kHeaderHeight);
	ResizeTheEditorBoard(theWindow);
}

void GetGrowSizeTheEditorWindow(Rect *sizeRect)
{
	GetGrowSizeTheLists(sizeRect, DEAD_SPACE_H, DEAD_SPACE_V, HEADER_SPACE);
}

void CopybitsTheEditorWindow(WindowRef theWindow, WindowRef offscreenWindowRef, Boolean active,
	Boolean drawGrowIconLines)
{
	Rect			tempRect;
	RgnHandle		copyRgn, listRgn;
	ListHandle		groupList, boardList;
	Rect			groupListRect, boardListRect;
	
	groupList=GetGroupListHandle();
	boardList=GetBoardListHandle();
	GetGroupListRect(&groupListRect);
	GetBoardListRect(&boardListRect);
	
	copyRgn=NewRgn();
	listRgn=NewRgn();
	RectRgn(copyRgn, &(theWindow->portRect));
	RectRgn(listRgn, &groupListRect);
	InsetRgn(listRgn, -4, -4);
	DiffRgn(copyRgn, listRgn, copyRgn);
	SetEmptyRgn(listRgn);
	RectRgn(listRgn, &boardListRect);
	InsetRgn(listRgn, -4, -4);
	DiffRgn(copyRgn, listRgn, copyRgn);
	
	CopyBits(	&(offscreenWindowRef->portBits),
				&(theWindow->portBits),
				&(offscreenWindowRef->portRect),
				&(theWindow->portRect), 0, copyRgn);
	
	if (active)
	{
		DrawGrowIconNoLines(theWindow, drawGrowIconLines);
	}
	else
	{
		tempRect.bottom=theWindow->portRect.bottom;
		tempRect.right=theWindow->portRect.right;
		tempRect.left=tempRect.right-kGrowBoxSize+1;
		tempRect.top=tempRect.bottom-kGrowBoxSize+1;
		EraseRect(&tempRect);
	}
	
	UpdateEditorLists(theWindow);
	
	DisposeRgn(copyRgn);
	DisposeRgn(listRgn);
}

void DrawTheEditorWindow(WindowRef theWindow, short theDepth)
{
	RGBColor		oldForeColor, oldBackColor;
	GrafPtr			curPort;
	Rect			groupListRect;
	
	if (theDepth>2)
	{
		GetForeColor(&oldForeColor);
		GetBackColor(&oldBackColor);
	}
	
	GetPort(&curPort);
	EraseRect(&(curPort->portRect));
	
	if (kHeaderHeight>0)
	{
		DrawEditorHeader(theWindow, theDepth);
	}
	
	GetGroupListRect(&groupListRect);
	DrawSoftWindowBorder(theWindow, theDepth, groupListRect.bottom+DEAD_SPACE_V, TRUE);
	DrawTheEditorBoard(theWindow, theDepth);
	
	if (theDepth>2)
	{
		RGBForeColor(&oldForeColor);
		RGBBackColor(&oldBackColor);
	}
}

void SetTheEditorMessage(WindowRef theWindow, short messageIndex, Boolean redraw)
{
	Str255			theStr;
	
	if ((messageIndex==kFreeTilesMessage) && (gCurrentBoardRec.numRows==0))
		messageIndex=kBlankMessage;
	if (messageIndex>0)
		GetIndString(gTheMessage, kEditorMessagesID, messageIndex);
	if (messageIndex==kFreeTilesMessage)
	{
		NumToString(FreeTiles(), theStr);
		AppendStr255(gTheMessage, theStr);
		AppendStr255(gTheMessage, "\p of ");
		NumToString(gCurrentBoardRec.numRows*gCurrentBoardRec.numColumns, theStr);
		AppendStr255(gTheMessage, theStr);
	}
	gTheMessageID=messageIndex;
	if (redraw)
		DrawTheEditorMessage(theWindow, FALSE);
}

void UpdateEditorButtons(WindowRef theWindow, Boolean onlyIfChanged)
{
	short			i;
	short			theDepth;
	
	if (!gShowToolbar)
		return;
	
	theDepth=GetWindowDepth(theWindow);
	for (i=0; i<NUM_BUTTONS; i++)
	{
		if ((!onlyIfChanged) || (ButtonIsDimmedQQ(theWindow, i)!=gOldButtonState[i]))
			Draw3DButton(&gButtonRect[i], 0L, gButtonIcon[i], theDepth, kSquareButton+
				(ButtonIsDimmedQQ(theWindow, i) ? kButtonIsDimmed : 0));
	}
}

short GetEditorHeaderHeight(void)
{
	return kHeaderHeight;
}

void RememberEditorButtonState(WindowRef theWindow)
{
	short			i;
	
	for (i=0; i<NUM_BUTTONS; i++)
		gOldButtonState[i]=ButtonIsDimmedQQ(theWindow, i);
}

void ResizeTheEditorBoard(WindowRef theWindow)
{
	short			width, height, maxTileHeight, maxTileWidth;
	Rect			groupListRect;
	
	if (gCurrentBoardRec.numRows==0)
		return;
	
	width=GetWindowWidth(theWindow)-DEAD_SPACE_H*2-38;
	GetGroupListRect(&groupListRect);
	height=GetWindowHeight(theWindow)-groupListRect.bottom-DEAD_SPACE_V*2-38;
	maxTileWidth=width/gCurrentBoardRec.numColumns;
	maxTileHeight=height/gCurrentBoardRec.numRows;
	gEditorTileSize=(maxTileWidth<maxTileHeight) ? maxTileWidth : maxTileHeight;
	gEditorBoardTopLeft.v=groupListRect.bottom+DEAD_SPACE_V*2;
	gEditorBoardTopLeft.h=DEAD_SPACE_H;
	gEditorBoardTopLeft.h+=
		(GetWindowWidth(theWindow)-(gEditorTileSize*gCurrentBoardRec.numColumns+28)-DEAD_SPACE_H*2)/2;
	gEditorBoardTopLeft.v+=
		(GetWindowHeight(theWindow)-groupListRect.bottom-DEAD_SPACE_V*3-
			(gEditorTileSize*gCurrentBoardRec.numRows+28))/2;
	InvalRect(&(theWindow->portRect));
}

/* the rest of these are internal to pent editor window.c */

static	void DrawTheEditorBoard(WindowRef theWindow, short theDepth)
{
	short			numRows, numCols, i, j;
	Rect			groupListRect;
	RGBColor		oldForeColor;
	
	if (theDepth>2)
	{
		GetForeColor(&oldForeColor);
		RGBForeColor(&gColor[kBlack]);
	}
	
	GetGroupListRect(&groupListRect);
	MoveTo(0, groupListRect.bottom+DEAD_SPACE_V);
	Line(GetWindowWidth(theWindow), 0);
	
	if (theDepth>2)
	{
		RGBForeColor(&gColor[kWhite]);
		MoveTo(0, groupListRect.bottom+DEAD_SPACE_V+1);
		Line(GetWindowWidth(theWindow), 0);
		RGBForeColor(&gColor[kLightGray]);
		MoveTo(0, groupListRect.bottom+DEAD_SPACE_V-1);
		Line(GetWindowWidth(theWindow), 0);
		MoveTo(GetWindowWidth(theWindow)-1, groupListRect.bottom+DEAD_SPACE_V-1);
		LineTo(GetWindowWidth(theWindow)-1, kHeaderHeight);
		RGBForeColor(&oldForeColor);
	}
	
	if (gCurrentBoardRec.numRows>0)
	{
		numRows=gCurrentBoardRec.numRows;
		numCols=gCurrentBoardRec.numColumns;
		DrawBoardBorder(theDepth, numRows, numCols, gEditorTileSize, gEditorTileSize,
			gEditorBoardTopLeft.h, gEditorBoardTopLeft.v);
		for (j=0; j<numCols; j++)
			for (i=0; i<numRows; i++)
				if (gCurrentBoardRec.boardData[i][j]!=kEmptyTile)
					DrawOneSpecialTile(theWindow, theDepth, gCurrentBoardRec.boardData, i, j,
						gEditorTileSize, gEditorTileSize, gEditorBoardTopLeft.h,
						gEditorBoardTopLeft.v, FALSE);
	}
}

static	void DrawEditorHeader(WindowRef theWindow, short theDepth)
{
	Rect			tempRect;
	RGBColor		background4={32767, 32767, 32767};
	RGBColor		background8={30802, 30802, 30802};
	RGBColor		oldForeColor, oldBackColor;
	short			width;
	short			headerHeight;
	GrafPtr			curPort;
	
	headerHeight=kHeaderHeight;
	
	GetPort(&curPort);
	tempRect=curPort->portRect;
	tempRect.bottom=tempRect.top+headerHeight-1;
	
	if (theDepth>2)
	{
		GetForeColor(&oldForeColor);
		GetBackColor(&oldBackColor);
		RGBForeColor((theDepth==4) ? &background4 : &background8);
		PaintRect(&tempRect);
		RGBForeColor(&oldForeColor);
		RGBBackColor(&oldBackColor);
	}
	else
	{
		EraseRect(&tempRect);
	}
	
	width=curPort->portRect.right-curPort->portRect.left;
	MoveTo(0, headerHeight-1);
	Line(width, 0);
	
	UpdateEditorButtons(theWindow, FALSE);
	
	if (gShowMessageBox)
		DrawTheEditorMessage(theWindow, TRUE);
}

static	Boolean ButtonIsDimmedQQ(WindowRef theWindow, short buttonNum)
{
	switch (buttonNum)
	{
		case kNewGroupButton:
		case kMessageButton:
		case kToolbarButton:
			return FALSE;
		case kNewBoardButton:
			return (GetHighlightedGroup()<0);
		case kEditButton:
		case kDeleteButton:
			return (GroupListActiveQQ() ?
				(GetHighlightedGroup()<0) :
				((GetHighlightedBoard()<0) || (HighlightedBoardLockedQQ())));
		case kClearButton:
			return (GetHighlightedBoard()<0) || (HighlightedBoardLockedQQ());
		case kSaveButton:
			return !WindowIsModifiedQQ(theWindow);
		case kLockButton:
			return (GetHighlightedBoard()<0);
	}
	
	return TRUE;
}

static	short FreeTiles(void)
{
	short			i, j, total;
	
	total=0;
	for (i=0; i<gCurrentBoardRec.numRows; i++)
		for (j=0; j<gCurrentBoardRec.numColumns; j++)
			if (gCurrentBoardRec.boardData[i][j]==kEmptyTile)
				total++;
	return total;
}

static	void DrawTheEditorMessage(WindowRef theWindow, Boolean drawFrame)
{
	Rect			tempRect;
	short			headerHeight;
	
	headerHeight=kHeaderHeight;
	tempRect=theWindow->portRect;
	tempRect.right-=DEAD_SPACE_H;
	tempRect.left+=DEAD_SPACE_H;
	tempRect.bottom=tempRect.top+headerHeight-DEAD_SPACE_H;
	tempRect.top=tempRect.bottom-MESSAGE_HEIGHT;
	if (drawFrame)
	{
		DrawTheShadowBox(tempRect, TRUE);
		InsetRect(&tempRect, 3, 3);
	}
	else
	{
		InsetRect(&tempRect, 3, 3);
		EraseRect(&tempRect);
	}
	
	if (gTheMessageID>0)
	{
		MoveTo(tempRect.left+5, tempRect.bottom-4);
		TextFont(geneva);
		TextSize(9);
		TextMode(srcOr);
		DrawString(gTheMessage);
	}
}
