#include "pent graphics.h"
#include "pent globals.h"
#include "environment.h"
#include "window layer.h"
#include "memory layer.h"

short			gColorPercent[kNumColors]={100,93,66,46,26,13,6,0};
RGBColor		gColor[kNumColors];
RGBColor		gTileColor[kNumTileColors];
PixPatHandle	gTilePattern[kNumTilesPatterns];

static	RgnHandle AllocateAcceptableDrawRegion(unsigned char boardData[20][20], short row,
	short col, short tileWidth, short tileHeight, short leftMargin, short topMargin,
	Boolean isRealPiece, Boolean matchingLeft, Boolean matchingTop, Boolean matchingRight,
	Boolean matchingBottom);

void RefreshTiles(WindowRef theWindow, Point *refresh, short numToRefresh, Boolean removingPiece,
	Boolean refreshFloat)
{
	short			refreshIndex;
	MyOffscreenPtr	offscreenWorldPtr;
	short			theDepth;
	
	if (removingPiece)
		SetWindowOffscreenNeedsUpdate(theWindow, TRUE);
	
	if (!WindowOffscreenNeedsUpdateQQ(theWindow))
	{	/* we can get away with just redrawing the stale tiles */
		theDepth=GetWindowDepth(theWindow);
		offscreenWorldPtr=GetWindowPermanentOffscreenWorld(theWindow);
		SetPortToOffscreenWindow(offscreenWorldPtr);
		for (refreshIndex=0; refreshIndex<numToRefresh; refreshIndex++)
		{
			DrawOneTile(theWindow, theDepth, gBoard, refresh[refreshIndex].h, refresh[refreshIndex].v,
				kTileWidth, kTileHeight, kLeftMargin, kTopMargin);
		}
		SetPortToOnscreenWindow(theWindow, offscreenWorldPtr);
		CopyBits(&(offscreenWorldPtr->offscreenPtr->portBits), &(theWindow->portBits),
			&(offscreenWorldPtr->offscreenPtr->portRect), &(theWindow->portRect), 0, 0L);
	}
	else
	{
		gCustomCursor=TRUE;
		UpdateTheWindow(theWindow);
		gCustomCursor=FALSE;
	}
	if (refreshFloat)
		UpdateTheWindow(GetIndWindowRef(kFloatingWindow));
}

void DrawOneSpecialTile(WindowRef theWindow, short theDepth, unsigned char boardData[20][20],
	short row, short col, short tileWidth, short tileHeight, short leftMargin, short topMargin,
	Boolean updateOffscreen)
{
	RgnHandle		oldClipRgn, theRgn;
	MyOffscreenPtr	offscreenWorldPtr;
	short			index;
	RGBColor		oldForeColor;
	
	if (theDepth>2)
		GetForeColor(&oldForeColor);
	
	theRgn=AllocateAcceptableDrawRegion(boardData, row, col, tileWidth, tileHeight, leftMargin,
		topMargin, FALSE, FALSE, FALSE, FALSE, FALSE);
	
	oldClipRgn=NewRgn();
	GetClip(oldClipRgn);
	SetClip(theRgn);
	
	index=boardData[row][col];
	if (theDepth>2)
	{
		RGBForeColor(&gColor[	(index==kSelectedTile) ? kVeryMediumGray :
								(index==kEmptyTile) ? kVeryLightGray :
								(index==kNoTile) ? kBlack : 0]);
	}
	else
	{
		switch (index)
		{
			case kSelectedTile:
				PenPat(&qd.gray);
				break;
			case kEmptyTile:
				ForeColor(whiteColor);
				break;
			case kNoTile:
				ForeColor(blackColor);
				break;
		}
	}
	
	PaintRect(&(theWindow->portRect));

	if (theDepth>2)
		RGBForeColor(&oldForeColor);
	else
	{
		PenNormal();
		ForeColor(blackColor);
	}
	
	SetClip(oldClipRgn);
	DisposeRgn(theRgn);
	DisposeRgn(oldClipRgn);
	
	if (updateOffscreen)
	{
		offscreenWorldPtr=GetWindowPermanentOffscreenWorld(theWindow);
		SetPortToOffscreenWindow(offscreenWorldPtr);
		DrawOneSpecialTile(theWindow, theDepth, boardData, row, col, tileWidth, tileHeight,
			leftMargin, topMargin, FALSE);
		SetPortToOnscreenWindow(theWindow, offscreenWorldPtr);
	}
}

void DrawOneTile(WindowRef theWindow, short theDepth, unsigned char boardData[20][20],
	short row, short col, short tileWidth, short tileHeight, short leftMargin, short topMargin)
{
	short			i, tileTop, tileLeft, index;
	Rect			theRect;
	short			tweak;
	Boolean			matchingTop, matchingBottom, matchingLeft, matchingRight;
	RgnHandle		theRgn, oldClipRgn;
	RGBColor		oldForeColor;
	
	index=boardData[row][col];
	if (index>kSelectedPiece)
	{
		DrawOneSpecialTile(theWindow, theDepth, boardData, row, col, tileWidth, tileHeight,
			leftMargin, topMargin, FALSE);
		return;
	}
	
	if (theDepth>2)
		GetForeColor(&oldForeColor);
	
	tileTop=topMargin+3+1+5+1+(row*tileHeight);
	tileLeft=leftMargin+8+3+1+5+1+(col*tileWidth);
	
	matchingTop=(row>0) && (boardData[row-1][col]==index);
	matchingBottom=(row<gNumRows-1) && (boardData[row+1][col]==index);
	matchingLeft=(col>0) && (boardData[row][col-1]==index);
	matchingRight=(col<gNumCols-1) && (boardData[row][col+1]==index);
	
	theRgn=AllocateAcceptableDrawRegion(boardData, row, col, tileWidth, tileHeight, leftMargin,
		topMargin, TRUE, matchingLeft, matchingTop, matchingRight, matchingBottom);
	oldClipRgn=NewRgn();
	GetClip(oldClipRgn);
	SetClip(theRgn);
	
	tweak=0;
	if (matchingTop)
		tweak++;
	theRect.top=tileTop-3-tweak;
	theRect.bottom=theRect.top+tileHeight-1+tweak;
	tweak=0;
	if (matchingRight)
		tweak++;
	theRect.left=tileLeft+4;
	theRect.right=theRect.left+tileWidth-1+tweak;
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kLeftEdge]);
	else
	{
		ForeColor(blackColor);
		PenPat(&qd.gray);
	}
	tweak=0;
	if (matchingTop)
		tweak++;
	for (i=0; i<4; i++)
	{
		MoveTo(tileLeft+i, tileTop-i-tweak);
		Line(0, tileHeight-2+tweak);
	}
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kBottomEdge]);
	else
	{
		PenNormal();
		ForeColor(blackColor);
	}
	tweak=0;
	if (matchingRight)
		tweak++;
	for (i=0; i<4; i++)
	{
		MoveTo(tileLeft+i+1, tileTop+tileHeight-2-i);
		Line(tileWidth-2+tweak, 0);
	}
	
	if (theDepth>2)
		PenPixPat(gTilePattern[index]);
	else
	{
		if (index==kSelectedPiece)
		{
			ForeColor(blackColor);
			PenPat(&qd.gray);
		}
		else
			ForeColor(whiteColor);
	}
	FrameRect(&theRect);
	
	if (matchingTop)
	{
		theRect.top++;
	}
	if (matchingRight)
	{
		theRect.right--;
		FrameRect(&theRect);
	}
	PenNormal();
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kRidgeLight]);
	else
		ForeColor(whiteColor);
	InsetRect(&theRect, 1, 1);
	MoveTo(theRect.left, theRect.bottom-1+(matchingBottom ? 2 : 0));
	LineTo(theRect.left, theRect.top-(matchingTop ? 2 : 0));
	MoveTo(theRect.left-(matchingLeft ? 2 : 0), theRect.top);
	LineTo(theRect.right-1+(matchingRight ? 2 : 0), theRect.top);
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kRidgeDark]);
	else
		ForeColor(blackColor);
	MoveTo(theRect.right-1, theRect.top-(matchingTop ? 3 : 0)+(matchingRight ? 1 : 0));
	LineTo(theRect.right-1, theRect.bottom-1+(matchingBottom ? 2 : 0));
	MoveTo(theRect.right-1+(matchingRight ? 2 : 0), theRect.bottom-1);
	LineTo(theRect.left+1-(matchingLeft ? 2 : 0), theRect.bottom-1);
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kRidge]);
	else
		PenPat(&qd.gray);
	InsetRect(&theRect, 1, 1);
	PenSize(2, 2);
	MoveTo(theRect.left, theRect.bottom-2+(matchingBottom ? 2 : 0));
	LineTo(theRect.left, theRect.top-(matchingTop ? 3 : 0));
	MoveTo(theRect.left-(matchingLeft ? 2 : 0), theRect.top);
	LineTo(theRect.right-2+(matchingRight ? 3 : 0), theRect.top);
	MoveTo(theRect.right-2, theRect.top-(matchingTop ? 3 : 0));
	LineTo(theRect.right-2, theRect.bottom-2+(matchingBottom ? 2 : 0));
	MoveTo(theRect.right-2+(matchingRight ? 3 : 0), theRect.bottom-2);
	LineTo(theRect.left-(matchingLeft ? 2 : 0), theRect.bottom-2);
	PenNormal();
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kRidgeDark]);
	else
		ForeColor(blackColor);
	InsetRect(&theRect, 2, 2);
	MoveTo(theRect.left, theRect.bottom-1+(matchingBottom ? 5 : 0));
	LineTo(theRect.left, theRect.top-(matchingTop ? 5 : 0));
	MoveTo(theRect.left-(matchingLeft ? 5 : 0), theRect.top);
	LineTo(theRect.right-1+(matchingRight ? 5 : 0), theRect.top);
	
	if (theDepth>2)
		RGBForeColor(&gTileColor[kRidgeLight]);
	else
		ForeColor(whiteColor);
	MoveTo(theRect.right-1, theRect.top-(matchingTop ? 6 : 0)+(matchingRight ? 1 : 0));
	LineTo(theRect.right-1, theRect.bottom-1+(matchingBottom ? 5 : 0));
	MoveTo(theRect.right-1+(matchingRight ? 5 : 0), theRect.bottom-1);
	LineTo(theRect.left+1-(matchingLeft ? 5 : 0), theRect.bottom-1);
	
	if (theDepth>2)
		PenPixPat(gTilePattern[index]);
	else
	{
		if (index==kSelectedPiece)
		{
			ForeColor(blackColor);
			PenPat(&qd.gray);
		}
		else
			ForeColor(whiteColor);
	}
	InsetRect(&theRect, 1, 1);
	PaintRect(&theRect);
	if (matchingTop)
	{
		theRect.top=tileTop-3;
		theRect.bottom=tileTop+2;
		theRect.left=tileLeft+9;
		theRect.right=tileLeft+tileWidth-2;
		PaintRect(&theRect);
	}
	if (matchingBottom)
	{
		theRect.top=tileTop+tileHeight-9;
		theRect.left=tileLeft+9;
		theRect.bottom=tileTop+tileHeight-2;
		theRect.right=tileLeft+tileWidth-2;
		PaintRect(&theRect);
	}
	if (matchingLeft)
	{
		theRect.top=tileTop+2;
		theRect.left=tileLeft+5;
		theRect.bottom=tileTop+tileHeight-9;
		theRect.right=tileLeft+9;
		PaintRect(&theRect);
	}
	if (matchingRight)
	{
		theRect.top=tileTop+2;
		theRect.left=tileLeft+tileWidth-2;
		theRect.bottom=tileTop+tileHeight-9;
		theRect.right=tileLeft+tileWidth+2;
		PaintRect(&theRect);
	}
	PenNormal();
	if (theDepth>2)
		RGBForeColor(&oldForeColor);
	else
		ForeColor(blackColor);
	
	SetClip(oldClipRgn);
	DisposeRgn(theRgn);
	DisposeRgn(oldClipRgn);
}

void DrawBoardBorder(short theDepth, short numRows, short numColumns,
	short tileWidth, short tileHeight, short leftMargin, short topMargin)
{
	short			i, w, h;
	Rect			theRect;
	RGBColor		oldForeColor;
	
	if (theDepth>2)
		GetForeColor(&oldForeColor);
	
	w=(numColumns*tileWidth)-1;
	h=(numRows*tileHeight)-1;
	
	if (theDepth>2)
		RGBForeColor(&gColor[kMediumGray]);
	else
		PenPat(&qd.gray);
	for (i=0; i<8; i++)
	{
		MoveTo(leftMargin+i, topMargin+7-i);
		Line(0, h+3+1+5+1+1+5+1+3);
	}
	
	if (theDepth>2)
		RGBForeColor(&gColor[kBlack]);
	else
	{
		PenNormal();
		ForeColor(blackColor);
	}
	for (i=0; i<8; i++)
	{
		MoveTo(leftMargin+8-i, topMargin+3+1+5+1+h+1+5+1+3+i);
		Line(w+3+1+5+1+1+5+1+3-1, 0);
	}
	
	if (theDepth>2)
		RGBForeColor(&gColor[kMediumDarkGray]);
	else
		ForeColor(blackColor);
	theRect.top=topMargin;
	theRect.left=leftMargin+8;
	theRect.bottom=theRect.top+h+3+1+5+1+1+5+1+3;
	theRect.right=theRect.left+w+3+1+5+1+1+5+1+3+1;
	for (i=0; i<3; i++)
	{
		FrameRect(&theRect);
		InsetRect(&theRect, 1, 1);
	}
	
	if (theDepth>2)
		RGBForeColor(&gColor[kLightGray]);
	else
		ForeColor(whiteColor);
	MoveTo(leftMargin+8+3, topMargin+3+1+5+1+h+5+1);
	Line(0, -(5+1+h+1+5+1));
	Line(5+1+w+1+5+1+1, 0);
	
	if (theDepth>2)
		RGBForeColor(&gColor[kBlack]);
	else
		ForeColor(blackColor);
	Move(0, 1);
	Line(0, 5+1+h+1+5);
	Line(-(5+1+w+1+5+1), 0);
	
	if (theDepth>2)
		RGBForeColor(&gColor[kVeryMediumGray]);
	else
		PenPat(&qd.gray);
	theRect.top=topMargin+3+1;
	theRect.left=leftMargin+8+3+1;
	theRect.bottom=theRect.top+5+1+h+1+5;
	theRect.right=theRect.left+5+1+w+1+5+1;
	for (i=0; i<5; i++)
	{
		FrameRect(&theRect);
		InsetRect(&theRect, 1, 1);
	}
	
	if (theDepth>2)
		RGBForeColor(&gColor[kBlack]);
	else
	{
		PenNormal();
		ForeColor(blackColor);
	}
	MoveTo(leftMargin+8+3+1+5, topMargin+3+1+5+1+h+1);
	Line(0, -(1+h+1));
	Line(1+w+1, 0);
	
	if (theDepth>2)
		RGBForeColor(&gColor[kLightGray]);
	else
		ForeColor(whiteColor);
	Move(0, 1);
	Line(0, h+1);
	Line(-(1+w), 0);
	
	theRect.left=leftMargin+8+3+1+5+1;
	theRect.top=topMargin+3+1+5+1;
	theRect.right=theRect.left+w;
	theRect.bottom=theRect.top+h;
	
	if (theDepth>2)
		RGBForeColor(&gColor[kMediumGray]);
	else
		ForeColor(blackColor);
	MoveTo(theRect.left-1, theRect.top+h);
	for (i=0; i<numColumns; i++)
	{
		Move(tileWidth, -h);
		Line(0, h);
	}
	MoveTo(theRect.left+w, theRect.top-1);
	for (i=0; i<numRows; i++)
	{
		Move(-w, tileHeight);
		Line(w, 0);
	}
	
	if (theDepth>2)
		RGBForeColor(&gColor[kWhite]);
	else
		ForeColor(whiteColor);
	MoveTo(theRect.left, theRect.top+h);
	for (i=1; i<numColumns; i++)
	{
		Move(tileWidth, -h);
		Line(0, h);
	}
	MoveTo(theRect.left+w, theRect.top);
	for (i=1; i<numRows; i++)
	{
		Move(-w, tileHeight);
		Line(w, 0);
	}
	
	if (theDepth>2)
		RGBForeColor(&oldForeColor);
	else
		ForeColor(blackColor);
}

void GetRowColFromPoint(Point thePoint, short *row, short *col, short tileWidth, short tileHeight,
	short leftMargin, short topMargin)
{
	*col=(thePoint.h-leftMargin-(8+3+1+5+1))/tileWidth;
	*row=(thePoint.v-topMargin-(3+1+5+1))/tileHeight;
}

void DrawSoftWindowBorder(WindowRef theWindow, short theDepth, short topMargin, Boolean growIcon)
{
	Rect			theRect;
	RGBColor		oldForeColor;
	
	theRect=theWindow->portRect;
	theRect.top=topMargin;
	if (theDepth>2)
	{
		GetForeColor(&oldForeColor);
		RGBForeColor(&gColor[kVeryLightGray]);
		PaintRect(&theRect);
		
		RGBForeColor(&gColor[kLightGray]);
		MoveTo(theRect.left, theRect.bottom-1);
		LineTo(theRect.right-1, theRect.bottom-1);
		LineTo(theRect.right-1, theRect.top+1);
		
		RGBForeColor(&gColor[kWhite]);
		MoveTo(theRect.left, theRect.top);
		LineTo(theRect.right-1, theRect.top);
		
		if (growIcon)
		{
			RGBForeColor(&gColor[kLightGray]);
			MoveTo(theRect.right-16, theRect.bottom-16);
			Line(0, 15);
			MoveTo(theRect.right-16, theRect.bottom-16);
			Line(15, 0);
		}
		
		RGBForeColor(&oldForeColor);
	}
	else
	{
		EraseRect(&theRect);
	}
}

static	RgnHandle AllocateAcceptableDrawRegion(unsigned char boardData[20][20], short row,
	short col, short tileWidth, short tileHeight, short leftMargin, short topMargin,
	Boolean isRealPiece, Boolean matchingLeft, Boolean matchingTop, Boolean matchingRight,
	Boolean matchingBottom)
/* caller is responsible for disposing of returned region */
{
	register short	L, R, T, B;
	short			tileTop, tileLeft;
	Rect			theRect;
	Boolean			touchingLeft, touchingBottom, touchingBottomLeft;
	RgnHandle		theRgn, otherRgn;
	
	tileTop=topMargin+3+1+5+1+(row*tileHeight);
	tileLeft=leftMargin+8+3+1+5+1+(col*tileWidth);
	
	touchingLeft=((col>0) && (boardData[row][col-1]<=kSelectedPiece));
	touchingBottom=((row<gNumRows-1) && (boardData[row+1][col]<=kSelectedPiece));
	touchingBottomLeft=((col>0) && (row<gNumRows-1) && (boardData[row+1][col-1]<=kSelectedPiece));

	L=theRect.left=tileLeft+1;
	T=theRect.top=tileTop+1;
	R=theRect.right=theRect.left+tileWidth-2;
	B=theRect.bottom=theRect.top+tileHeight-2;
	
	theRgn=NewRgn();
	RectRgn(theRgn, &theRect);
	otherRgn=NewRgn();
	if (isRealPiece)
	{
		OpenRgn();
		{
			MoveTo(L, T);
			if (matchingTop)
				Line(0, -2);
			Line(0, -1);
			Line(3, -3);
			LineTo(R+4, T-(matchingTop ? 6 : 4));
			if (matchingTop && matchingRight)
				Line(0, 2);
			if (matchingRight)
				Line(1, 0);
			LineTo(R+(matchingRight ? 5 : 4), B);
			if (matchingRight)
				Line(-1, 0);
			Line(-4, 4);
			LineTo(R, T);
			LineTo(L, T);
		}
		CloseRgn(otherRgn);
		UnionRgn(theRgn, otherRgn, theRgn);
		SetEmptyRgn(otherRgn);
	}
	
	if (touchingLeft)
	{
		OpenRgn();
		{
			MoveTo(L, T-(matchingTop ? 4 : 2));
			LineTo(L+(matchingLeft ? 3 : 2), T-(matchingTop ? 4 : 2));
			LineTo(L+(matchingLeft ? 3 : 2), B-4);
			LineTo(L, B-(matchingLeft ? 1 : 2));
			LineTo(L, T-(matchingTop ? 4 : 2));
		}
		CloseRgn(otherRgn);
		DiffRgn(theRgn, otherRgn, theRgn);
		SetEmptyRgn(otherRgn);
	}
	
	if (touchingBottom)
	{
		OpenRgn();
		{
			MoveTo(R+4, B);
			LineTo(R+4, B-(matchingBottom ? 3 : 2));
			LineTo(L+(matchingBottom ? 2 : 3), B-(matchingBottom ? 3 : 2));
			LineTo(L+(matchingBottom ? -1 : 1), B);
			LineTo(R+4, B);
		}
		CloseRgn(otherRgn);
		DiffRgn(theRgn, otherRgn, theRgn);
		SetEmptyRgn(otherRgn);
	}
	
	if (touchingBottomLeft)
	{
		OpenRgn();
		{
			MoveTo(L, B);
			Line(touchingBottom ? 3 : 2, 0);
			Line(0, -2);
			Line(touchingBottom ? -3 : -2, 0);
			Line(0, 2);
		}
		CloseRgn(otherRgn);
		DiffRgn(theRgn, otherRgn, theRgn);
		SetEmptyRgn(otherRgn);
		
		OpenRgn();
		{
			MoveTo(L, B);
			Line(2, 0);
			Line(0, touchingLeft ? -3 : -2);
			Line(-2, 0);
			Line(0, touchingLeft ? 3 : 2);
		}
		CloseRgn(otherRgn);
		DiffRgn(theRgn, otherRgn, theRgn);
		SetEmptyRgn(otherRgn);
	}
	
	if (touchingBottom && touchingBottomLeft)
	{
		if (!matchingBottom && (boardData[row+1][col]!=boardData[row+1][col-1]))
		{
			OpenRgn();
			{
				MoveTo(L+2, B-2);
				Line(1, 0);
				Line(0, 1);
				Line(-1, 0);
				Line(0, -1);
			}
			CloseRgn(otherRgn);
			UnionRgn(theRgn, otherRgn, theRgn);
			SetEmptyRgn(otherRgn);
		}
	}

	if (touchingLeft && touchingBottomLeft)
	{
		if (!matchingLeft && (boardData[row][col-1]!=boardData[row+1][col-1]))
		{
			OpenRgn();
			{
				MoveTo(L+1, B-3);
				Line(1, 0);
				Line(0, 1);
				Line(-1, 0);
				Line(0, -1);
			}
			CloseRgn(otherRgn);
			UnionRgn(theRgn, otherRgn, theRgn);
			SetEmptyRgn(otherRgn);
		}
	}
	
	DisposeRgn(otherRgn);
	
	return theRgn;
}
