#include "pent load-save.h"
#include "pent main window.h"
#include "pent.h"
#include "pent globals.h"
#include "environment.h"
#include "error.h"
#include "file utilities.h"
#include "file interface.h"
#include "memory layer.h"
#include "resource layer.h"
#include "window layer.h"

#define kGameResType		'PGme'
#define kGameResID			200

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif
struct MySaveRec
{
	short			version;
	unsigned short	checksum;
	short			numRows;
	short			numColumns;
	short			numPlayed;
	short			numHighlighted;
	short			selectedPieceColor;
	short			unused;
	Boolean			pieceUsed[12];
	unsigned char	piecesPlayed[12];
	unsigned char	boardData[kBoardRowsMax][kBoardColumnsMax];
};
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

typedef struct MySaveRec		MySaveRec, *MySavePtr;

static	unsigned short ComputeChecksum(Ptr input, unsigned short len);

ErrorTypes LoadSaveDispatch(FSSpec *fs, Boolean isLoad, Boolean useOldFile)
/* handles errors, but returns error code for your convenience anyway */
{
	Boolean			alreadyExists;
	ErrorTypes		theError;
	WindowRef		mainWindowRef;
	
	theError=allsWell;
	
	if ((!isLoad) && ((mainWindowRef=GetIndWindowRef(kMainWindow))==0L))
		return userCancelErr;
	
	if (isLoad)
	{
		if (GetSourceFile(fs, SAVE_TYPE))
		{
			if ((theError=GetTheFile(fs))!=allsWell)
				HandleError(theError, FALSE, FALSE);
		}
	}
	else
	{
		useOldFile=useOldFile&&((fs->name)[0]!=0x00);
		alreadyExists=(FileExists(fs)==noErr);
		if (!useOldFile)
		{
			if (!GetDestFile(fs, &alreadyExists, SAVE_DIALOG_PROMPT))
				return userCancelErr;
		}
		
		if ((theError=SaveTheFile(fs, alreadyExists))!=allsWell)
		{
			FSpDelete(fs);
			fs->name[0]=0x00;
			HandleError(theError, FALSE, FALSE);
		}
	}
	
	return theError;
}

ErrorTypes SaveTheFile(FSSpec *saveFile, Boolean alreadyExists)
{
	short			oldRefNum, refNum, dataRefNum, i, j;
	Boolean			alreadyOpen;
	Handle			resHandle;
	OSErr			oe;
	MySaveRec		saveRec;
	unsigned short	actualChecksum;
	long			dataForkLength, dataForkOffset;
	Ptr				dataForkPtr;
	unsigned char	theChar;
	WindowRef		theWindow;
	
	oe=FSpCreate(saveFile, CREATOR, SAVE_TYPE, smSystemScript);
	FlushVol(0L, saveFile->vRefNum);
	if (!alreadyExists && (oe!=noErr))
		return kCantCreateFile;
	FSpCreateResFile(saveFile, CREATOR, SAVE_TYPE, smSystemScript);
	if (!alreadyExists && (ResError()!=noErr))
		return kCantCreateFile;
	FlushVol(0L, saveFile->vRefNum);
	if ((OpenTheResFile(saveFile, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return kCantOpenFileToSave;
	if (FSpOpenDF(saveFile, fsRdWrPerm, &dataRefNum))
	{
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		return kCantOpenFileToSave;
	}
	
	dataForkLength=gNumRows*(gNumCols+1);
	if ((dataForkPtr=NewPtrClear(dataForkLength))==0L)
	{
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		FSClose(dataRefNum);
		return kNoMemory;
	}
	if (SetEOF(dataRefNum, dataForkLength)!=noErr)
	{
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		FSClose(dataRefNum);
		return kDiskFull;
	}
	
	if (alreadyExists)
	{
		while ((resHandle=Get1IndResource(kGameResType, 1))!=0L)
		{
			RmveResource(resHandle);
			oe=ResError();
			if (oe!=noErr)
			{
				ReleaseResource(resHandle);
				CloseTheResFile(oldRefNum, refNum, alreadyOpen);
				FSClose(dataRefNum);
				return kCantWriteFile;
			}
			UpdateResFile(refNum);
			DisposeHandle(resHandle);	/* correct, this handle is no longer a resource */
			resHandle=0L;
		}
	}
	
	saveRec.version=SAVE_VERSION;
	saveRec.checksum=0;
	saveRec.numRows=gNumRows;
	saveRec.numColumns=gNumCols;
	saveRec.numPlayed=gNumPlayed;
	saveRec.numHighlighted=gNumHilited;
	saveRec.selectedPieceColor=gSelectedPieceColor;
	saveRec.unused=0;
	for (i=0; i<12; i++)
		saveRec.piecesPlayed[i]=gPiecesPlayed[i];
	for (i=0; i<12; i++)
		saveRec.pieceUsed[i]=gPieceUsed[i];
	for (i=0; i<kBoardRowsMax; i++)
		for (j=0; j<kBoardColumnsMax; j++)
			saveRec.boardData[i][j]=gBoard[i][j];
	actualChecksum=ComputeChecksum((Ptr)&saveRec, sizeof(MySaveRec));
	saveRec.checksum=actualChecksum;
	resHandle=NewHandle(sizeof(MySaveRec));
	Mymemcpy((Ptr)*resHandle, (Ptr)&saveRec, sizeof(MySaveRec));
	AddResource(resHandle, kGameResType, kGameResID, "\p");
	oe=ResError();
	if (oe==noErr)
	{
		WriteResource(resHandle);
		UpdateResFile(refNum);
	}
	CloseTheResFile(oldRefNum, refNum, alreadyOpen);
	SafeDisposeHandle(resHandle);	/* will DisposeHandle or ReleaseResource appropriately */
	if (oe!=noErr)
	{
		FSClose(dataRefNum);
		return kCantWriteFile;
	}
	
	/* the game file has been saved successfully; the rest of this is just for show */
	dataForkOffset=0L;
	for (i=0; i<gNumRows; i++)
	{
		for (j=0; j<=gNumCols; j++)
		{
			if (j==gNumCols)
				theChar='\r';
			else
			{
				switch (gBoard[i][j])
				{
					case kSelectedPiece:
						theChar=gSelectedPieceColor+'A';
						break;
					case kNoTile:
						theChar='#';
						break;
					case kEmptyTile:
						theChar='.';
						break;
					case kSelectedTile:
						theChar=',';
						break;
					default:
						theChar=gBoard[i][j]+'A';
						break;
				}
			}
			*((unsigned char*)dataForkPtr+(dataForkOffset++))=theChar;
		}
	}
	
	oe=FSWrite(dataRefNum, &dataForkLength, dataForkPtr);
	FSClose(dataRefNum);
	FlushVol(0L, saveFile->vRefNum);
	if (oe!=noErr)
		return kCantWriteFile;
	
	theWindow=GetIndWindowRef(kMainWindow);
	SetWTitle(theWindow, saveFile->name);
	SetWindowTitle(theWindow, saveFile->name);
	SetWindowFS(theWindow, *saveFile);
	SetWindowIsModified(theWindow, FALSE);
	
	return allsWell;
}

ErrorTypes GetTheFile(FSSpec *saveFile)
{
	short			oldRefNum, refNum, i, j, saveVersion;
	Boolean			alreadyOpen;
	Handle			resHandle;
	MySaveRec		saveRec;
	unsigned short	savedChecksum;
	WindowRef		theWindow;
	
	if ((OpenTheResFile(saveFile, &oldRefNum, &refNum, &alreadyOpen, FALSE))!=noErr)
		return kCantOpenFileToLoad;
	
	if ((resHandle=Get1Resource(kGameResType, kGameResID))==0L)
	{
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		return kCantLoadFile;
	}
	
	Mymemcpy((Ptr)&saveVersion, (Ptr)*resHandle, sizeof(short));
	if (saveVersion!=SAVE_VERSION)
	{
		ReleaseResource(resHandle);
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		return kBadVersion;
	}
	
	if (GetHandleSize(resHandle)!=sizeof(MySaveRec))
	{
		ReleaseResource(resHandle);
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		return kBadChecksum;
	}
	
	Mymemcpy((Ptr)&saveRec, (Ptr)*resHandle, sizeof(MySaveRec));
	ReleaseResource(resHandle);
	
	savedChecksum=saveRec.checksum;
	saveRec.checksum=0;
	if (savedChecksum!=ComputeChecksum((Ptr)&saveRec, sizeof(MySaveRec)))
	{
		CloseTheResFile(oldRefNum, refNum, alreadyOpen);
		return kBadChecksum;
	}
	
	gNumRows=saveRec.numRows;
	gNumCols=saveRec.numColumns;
	gNumPlayed=saveRec.numPlayed;
	gNumHilited=saveRec.numHighlighted;
	gSelectedPieceColor=saveRec.selectedPieceColor;
	for (i=0; i<12; i++)
		gPiecesPlayed[i]=saveRec.piecesPlayed[i];
	for (i=0; i<12; i++)
		gPieceUsed[i]=saveRec.pieceUsed[i];
	for (i=0; i<kBoardRowsMax; i++)
		for (j=0; j<kBoardColumnsMax; j++)
			gBoard[i][j]=saveRec.boardData[i][j];
	
	if ((theWindow=GetIndWindowRef(kMainWindow))!=0L)
	{	/* may happen if gAskBeforeClosing is FALSE */
		SetWindowIsModified(theWindow, FALSE);
		CloseTheWindow(theWindow);
	}
	SetNewMainWindowTitle(saveFile->name);
	gThisGameTileSize=gTileSize;
	gCustomCursor=TRUE;
	OpenTheIndWindow(kMainWindow, kAlwaysOpenNew);
	if ((theWindow=GetIndWindowRef(kMainWindow))!=0L)
		SetWindowFS(theWindow, *saveFile);
	StartGame();	/* really only plays the sound or handles memory error */
	
	return allsWell;
}

static	unsigned short ComputeChecksum(Ptr input, unsigned short len)
{
	unsigned short	i;
	Boolean			shiftedOut;
	unsigned short	checksum;
	
	checksum=0;
	for (i=0; i<len; i++)
	{
		shiftedOut=(checksum&0x8000==0) ? FALSE : TRUE;
		checksum+=*((unsigned char*)((long)input+i));
		checksum<<=1;
		if (shiftedOut)
			checksum^=0xdead;
	}
	
	return checksum;
}
