#include "pent main window.h"
#include "pent graphics.h"
#include "pent meat.h"
#include "pent message.h"
#include "pent endgame.h"
#include "pent globals.h"
#include "cursor layer.h"
#include "environment.h"
#include "error.h"
#include "key layer.h"
#include "memory layer.h"
#include "save dialog.h"
#include "sound layer.h"
#include "window layer.h"
#include "program globals.h"

static	long			gLastClickTime=0L;
static	Str31			gMainWindowTitleStr;

void SetupTheMainWindow(WindowRef theWindow)
{
	short			w, h, floatSpace;
	FSSpec			fs;
	Rect			screenRect;
	Boolean			fits;
	Point			topLeft;
	
	GetMainScreenBounds(&screenRect);
	floatSpace=gShowFloat ? (gAnchorFloat ? 52 : 64) : 0;
	do
	{
		w=kLeftMargin+8+3+1+5+1+(gNumCols*kTileWidth)-1+1+5+1+3+kRightMargin;
		h=kTopMargin+3+1+5+1+(gNumRows*kTileHeight)-1+1+5+1+3+8+kBottomMargin;
		fits=((w<(screenRect.right-screenRect.left-20)) &&
				(h<(screenRect.bottom-screenRect.top-38-floatSpace)));
		if (!fits && gAutoSize)
			gThisGameTileSize--;
	}
	while (!fits && gAutoSize && (gThisGameTileSize>15));
	SetWindowHeight(theWindow, h);
	SetWindowWidth(theWindow, w);
	SetWindowAttributes(theWindow, kHasCloseBoxMask+kHasDocumentTitlebarMask);
	SetWindowMaxDepth(theWindow, 8);
	SetWindowDepth(theWindow, 8);
	SetWindowIsFloat(theWindow, FALSE);
	SetWindowTitle(theWindow, gMainWindowTitleStr);
	SetWindowAutoCenter(theWindow, FALSE);
	SetWindowIsModified(theWindow, FALSE);
	SetWindowHasPermanentOffscreenWorld(theWindow, TRUE);
	fs.name[0]=0x00;
	SetWindowFS(theWindow, fs);
	topLeft.h=screenRect.left+
		(((screenRect.right-screenRect.left)-GetWindowWidth(theWindow))/2);
	topLeft.v=9+screenRect.top+floatSpace+
		(((screenRect.bottom-screenRect.top-floatSpace)-GetWindowHeight(theWindow))/2);
	SetWindowTopLeft(theWindow, topLeft);
}

void OpenTheMainWindow(WindowRef theWindow)
{
	Boolean			depthChanged;
	MyOffscreenPtr	offscreenWorldPtr;

	if ((offscreenWorldPtr=AllocateOffscreenWorld(theWindow, &depthChanged))==0L)
		HandleError(kNoMemoryAndQuitting, TRUE, FALSE);
	SetPortToOnscreenWindow(theWindow, offscreenWorldPtr);
	SetWindowHasPermanentOffscreenWorld(theWindow, TRUE);
	SetWindowPermanentOffscreenWorld(theWindow, offscreenWorldPtr);
	SetWindowOffscreenNeedsUpdate(theWindow, TRUE);
	if (gShowFloat)
		OpenTheIndWindow(kFloatingWindow, kOpenOldIfPossible);
}

Boolean CloseTheMainWindow(WindowRef theWindow)
{
	return (gAskBeforeClosing ?
			(DisplaySaveAlert(theWindow, "\pthe game in progress", "\p",
				gDone ? "\pquitting" : "\pclosing")!=kUserCanceled) :
			TRUE);
}

void DisposeTheMainWindow(WindowRef theWindow)
{
	MyOffscreenPtr	offscreenWorldPtr;
	
	if ((offscreenWorldPtr=GetWindowPermanentOffscreenWorld(theWindow))!=0L)
		DisposeOffscreenWorld(offscreenWorldPtr);
	if (gShowFloat)
	{
		CloseTheWindow(GetIndWindowRef(kFloatingWindow));
		gShowFloat=TRUE;
	}
}

void ActivateTheMainWindow(void)
{
	WindowRef		theWindow;
	
	if ((theWindow=GetIndWindowRef(kFloatingWindow))==0L)
		return;
	
	if (gShowFloat)
		MyShowWindow(theWindow);
}

void DeactivateTheMainWindow(void)
{
	WindowRef		theWindow;
	
	if ((theWindow=GetIndWindowRef(kFloatingWindow))==0L)
		return;
	
	if (gShowFloat && GetWindowHilite(theWindow))
		MyHideWindow(theWindow);
}

void KeyPressedInMainWindow(WindowRef theWindow, unsigned char charPressed)
{
	if ((charPressed==key_Return) || (charPressed==key_Enter))
	{
		if (gNumHilited==5)
		{
			if (gSelectedPieceColor==kSelectedTile)
				PlaceUnknownPiece(theWindow);
		}
		else if (gNumHilited>0)
		{
			DisplayMessage(eNotAPiece);
		}
	}
	if (charPressed==key_Delete)
	{
	 	if ((gNumHilited==5) && (gSelectedPieceColor!=kSelectedTile))
			RemovePiece(theWindow);
		else if (gNumHilited>0)
			UndoInMainWindow(theWindow);
	}
}

void MouseClickedInMainWindow(WindowRef theWindow, Point mouseLoc)
{
	Rect			boardRect;
	short			i, j, iter, utter, whichWay, width, height, theDepth;
	long			tickTime;
	Point			refresh[5];
	short			refreshIndex;
	
	tickTime=LMGetTicks();
	width=GetWindowWidth(theWindow);
	height=GetWindowHeight(theWindow);
	theDepth=GetWindowDepth(theWindow);
	
	boardRect.left=kLeftMargin+8+3+1+5+1+1;
	boardRect.top=kTopMargin+3+1+5+1+1;
	boardRect.right=boardRect.left+(gNumCols*kTileWidth)-1;
	boardRect.bottom=boardRect.top+(gNumRows*kTileHeight)-1;
	
	if (!PtInRect(mouseLoc, &boardRect)) return;
	
	whichWay=0;
	
	while (StillDown())
	{
		GetMouse(&mouseLoc);
		if (PtInRect(mouseLoc, &boardRect))
		{
			GetRowColFromPoint(mouseLoc, &i, &j, kTileWidth, kTileHeight, kLeftMargin, kTopMargin);

			switch (gBoard[i][j])
			{
				case kSelectedTile:
					if (whichWay>=0)
					{
						gNumHilited--;
						gBoard[i][j]=kEmptyTile;
						DrawOneSpecialTile(theWindow, theDepth, gBoard, i, j, kTileWidth,
							kTileHeight, kLeftMargin, kTopMargin, TRUE);
						whichWay=1;
						SetWindowIsModified(theWindow, TRUE);
					}
					break;
				case kEmptyTile:
					if ((whichWay<=0) && (gSelectedPieceColor==kSelectedTile))
					{
						gNumHilited++;
						gBoard[i][j]=kSelectedTile;
						DrawOneSpecialTile(theWindow, theDepth, gBoard, i, j, kTileWidth,
							kTileHeight, kLeftMargin, kTopMargin, TRUE);
						whichWay=-1;
						SetWindowIsModified(theWindow, TRUE);
					}
					break;
				case kSelectedPiece:
					if (tickTime-gLastClickTime<LMGetDoubleTime())
						RemovePiece(theWindow);
					else
						UndoInMainWindow(theWindow);
					gLastClickTime=0L;
					return;
					break;
				default:
				{
					if ((whichWay==0) && (gNumHilited==0) && (gBoard[i][j]!=kNoTile))
					{
						gSelectedPieceColor=gBoard[i][j];
						refreshIndex=0;
						for (iter=0; iter<gNumRows; iter++)
						{
							for (utter=0; utter<gNumCols; utter++)
							{
								if (gBoard[iter][utter]==gSelectedPieceColor)
								{
									gBoard[iter][utter]=kSelectedPiece;
									refresh[refreshIndex].h=iter;
									refresh[refreshIndex++].v=utter;
								}
							}
						}
						gNumHilited=5;
						RefreshTiles(theWindow, refresh, 5, FALSE, FALSE);
						gLastClickTime=LMGetTicks();
						SetWindowIsModified(theWindow, TRUE);
						return;
					}
					break;
				}
			}
		}
	}
}

void MouseUpInMainWindow(void)
{
	if (gLastClickTime!=0L)
		gLastClickTime=LMGetTicks();
}

void DrawTheMainWindow(WindowRef theWindow, short theDepth)
{
	short			row, col;
	RGBColor		oldForeColor, oldBackColor;
	
	if (theDepth>2)
	{
		GetForeColor(&oldForeColor);
		GetBackColor(&oldBackColor);
	}
	
	DrawSoftWindowBorder(theWindow, theDepth, 0, FALSE);
	DrawBoardBorder(theDepth, gNumRows, gNumCols, kTileWidth, kTileHeight, kLeftMargin, kTopMargin);
	for (col=0; col<gNumCols; col++)
	{
		for (row=gNumRows-1; row>=0; row--)
		{
			DrawOneTile(theWindow, theDepth, gBoard, row, col, kTileWidth, kTileHeight,
				kLeftMargin, kTopMargin);
			if (gCustomCursor)
				AnimateTheCursor();
		}
	}
	
	if (theDepth>2)
	{
		RGBForeColor(&oldForeColor);
		RGBBackColor(&oldBackColor);
	}
}

void UndoInMainWindow(WindowRef theWindow)
{
	short			i, j;
	short			refreshIndex, numToRefresh;
	Point			refresh[160];	/* needs to be as large as largest possible board (8 x 20) */
	
	refreshIndex=0;
	numToRefresh=gNumHilited;
	
	if ((gNumHilited>0) && (gSelectedPieceColor==kSelectedTile))
	{	/* some squares selected but no piece played yet, so de-hilite selected squares */
		gNumHilited=0;
		for (j=0; ((j<gNumCols) && (refreshIndex<numToRefresh)); j++)
		{
			for (i=gNumRows-1; ((i>=0) && (refreshIndex<numToRefresh)); i--)
			{
				if (gBoard[i][j]==kSelectedTile)
				{
					gBoard[i][j]=kEmptyTile;
					refresh[refreshIndex].h=i;
					refresh[refreshIndex++].v=j;
				}
			}
		}
		gSelectedPieceColor=kSelectedTile;
		RefreshTiles(theWindow, refresh, numToRefresh, FALSE, FALSE);
	}
	else if (gNumHilited>0)
	{	/* a piece is selected, so return it to normal */
		gNumHilited=0;
		for (j=0; ((j<gNumCols) && (refreshIndex<numToRefresh)); j++)
		{
			for (i=gNumRows-1; ((i>=0) && (refreshIndex<numToRefresh)); i--)
			{
				if (gBoard[i][j]==kSelectedPiece)
				{
					gBoard[i][j]=gSelectedPieceColor;
					refresh[refreshIndex].h=i;
					refresh[refreshIndex++].v=j;
				}
			}
		}
		gSelectedPieceColor=kSelectedTile;
		RefreshTiles(theWindow, refresh, numToRefresh, FALSE, FALSE);
	}
	else if (gNumPlayed)
	{	/* undo the last piece placed */
		gNumPlayed--;
		gPieceUsed[gPiecesPlayed[gNumPlayed]] = FALSE;
		for (j=0; ((j<gNumCols) && (refreshIndex<5)); j++)
		{
			for (i=gNumRows-1; ((i>=0) && (refreshIndex<5)); i--)
			{
				if (gBoard[i][j]==gPiecesPlayed[gNumPlayed])
				{
					gBoard[i][j]=kEmptyTile;
					refresh[refreshIndex].h=i;
					refresh[refreshIndex++].v=j;
				}
			}
		}
		RefreshTiles(theWindow, refresh, 5, TRUE, TRUE);
		DoSound(sound_undo, FALSE);
	}
	SetWindowIsModified(theWindow, TRUE);
}

void SetNewMainWindowTitle(Str31 titleStr)
{
	Mymemcpy((Ptr)gMainWindowTitleStr, (Ptr)titleStr, titleStr[0]+1);
}

void DealWithChangedPieceSize(WindowRef theWindow)
{
	short			w, h, floatSpace;
	Point			topLeft;
	Rect			screenRect;
	MyOffscreenPtr	offscreenWorldPtr;
	
	if (theWindow==0L)
		return;
	if (gThisGameTileSize==gTileSize)
		return;
	
	gThisGameTileSize=gTileSize;
	w=kLeftMargin+8+3+1+5+1+(gNumCols*kTileWidth)-1+1+5+1+3+kRightMargin;
	h=kTopMargin+3+1+5+1+(gNumRows*kTileHeight)-1+1+5+1+3+8+kBottomMargin;
	MyHideWindow(theWindow);
	MySizeWindow(theWindow, w, h, TRUE);
	floatSpace=gShowFloat ? (gAnchorFloat ? 52 : 64) : 0;
	GetMainScreenBounds(&screenRect);
	topLeft.h=screenRect.left+
		(((screenRect.right-screenRect.left)-GetWindowWidth(theWindow))/2);
	topLeft.v=9+screenRect.top+floatSpace+
		(((screenRect.bottom-screenRect.top-floatSpace)-GetWindowHeight(theWindow))/2);
	SetWindowTopLeft(theWindow, topLeft);
	MyMoveWindow(theWindow, topLeft.h, topLeft.v, TRUE);
	if ((offscreenWorldPtr=GetWindowPermanentOffscreenWorld(theWindow))!=0L)
		DisposeOffscreenWorld(offscreenWorldPtr);
	OpenTheMainWindow(theWindow);
	SetPort(theWindow);
	EraseRect(&(theWindow->portRect));
	MyShowWindow(theWindow);
	UpdateTheWindow(theWindow);
}
